package mpi.eudico.client.annotator.interlinear.edit;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.ButtonGroup;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.gui.ClosableDialog;
import mpi.eudico.client.annotator.layout.InterlinearizationManager;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import nl.mpi.lexan.annotyzers.helpers.Information;
import nl.mpi.lexan.annotyzers.helpers.TierNodeType;
import nl.mpi.lexan.annotyzers.helpers.parameters.Parameter;


/**
 * A dialog to add configurations based on tier or type
 * 
 * @author aarsom
 * 
 */
@SuppressWarnings("serial")
public class AnnotConfigDialog extends ClosableDialog implements ActionListener{
	
	private final String SELECT = ElanLocale.getString("InterlinearAnnotyzerConfigDlg.ComboBoxDefaultString");	
	private final String NO_TIERS = ElanLocale.getString("InterlinearAnnotyzerConfigDlg.NoSource");		
	private final String COLUMN_NO = ElanLocale.getString("TranscriptionTable.Column.No");    
	
	private final int TIER_MODE = 0;
	private final int TYPE_MODE = 1;
	
	private InterlinearizationManager manager;	
	
	private JRadioButton tierRB;
	private JRadioButton typeRB;
	
	private JCheckBox overwriteCB;
	
	private JTable configsTable;
	private DefaultTableModel configsModel;
	
	private JComboBox annotSelCB;
	private JComboBox sourceCB;	
	private JComboBox destinationCB;
	
	private JButton removeButton;
	private JButton addButton;
    private JButton cancelButton;	
	
    //<SourceName, targetList>
	private Map<String, List<String>> typesMap;	
	
	//<SourceName, targetList>
	private Map<String, List<String>> tiersMap;	
	
	//<SourceName, targetList>
	private Map<String, List<String>> currentRowMap;
	private int currentMode = -1;
	
	//Details regarding the contents of the map
    //Integer: integer starts with 1 as a index
    //List<Object> contains 3 values : [0] <String> - annotyzer, 
	//[1]<List<String>> -targetTierList, [2]<String> - source
	private Map<Integer, List<Object>> typeConfigMap;
	private Map<Integer, List<Object>> tierConfigMap;
	
	// <annotyzerName, number of target tiers>
	private Map<String, Integer> targetTiersMap;	
	int maxTargetTiers = 1;
	
	private final int ANNOTYZER_INDEX = 1;
	private final int SOURCE_INDEX = 2;
	
	/**
	 * Constructor 
	 * 
	 * @param frame, the elan frame
	 * @param manager, the interlinearization manager, cannot be null
	 */
	public AnnotConfigDialog(mpi.eudico.client.annotator.ElanFrame2 frame,
			InterlinearizationManager manager){
        super(frame, ElanLocale.getString("InterlinearAnnotyzerConfigDlg.Title"), true);      
              
        typesMap =  new HashMap<String, List<String>>();
        tiersMap =  new HashMap<String, List<String>>();
        
        typeConfigMap =  new HashMap<Integer, List<Object>>();
        tierConfigMap =  new HashMap<Integer, List<Object>>();
        
        currentRowMap = new HashMap<String, List<String>>(); 
        
        targetTiersMap = new HashMap<String, Integer>();
        
        this.manager = manager;
        
        initComponents();   
		postInit();
    }
    
    /**
     * Pack, size and set location.
     */
    private void postInit() {
    	pack();       
        setResizable(true);
        setLocationRelativeTo(getParent());
    }
    
    /**
     * Initializes the components for ui.
     */
    private void initComponents() {      	
    	//initialize the components 
		tierRB = new JRadioButton(ElanLocale.getString("InterlinearAnnotyzerConfigDlg.Tiers"));			
		tierRB.addActionListener(this);
		
		typeRB = new JRadioButton(ElanLocale.getString("InterlinearAnnotyzerConfigDlg.Types"));
		typeRB.addActionListener(this);
		
		overwriteCB = new JCheckBox(ElanLocale.getString("InterlinearAnnotyzerConfigDlg.Overwrite"));
		
		sourceCB = new JComboBox();
		destinationCB = new JComboBox();
		annotSelCB = new JComboBox();
		
		ButtonGroup group = new ButtonGroup();
		group.add(tierRB);
		group.add(typeRB);
		
		removeButton = new JButton(ElanLocale.getString("FileChooser.Button.Remove"));
		removeButton.addActionListener(this);	
    	
        addButton = new JButton(ElanLocale.getString("Button.Add"));
        addButton.addActionListener(this);      
        
        cancelButton = new JButton(ElanLocale.getString("Button.Cancel"));
        cancelButton.addActionListener(this); 
		
        // get information for all annotyzers
		if(manager != null){
			List<Information> tyzersList = manager.getMediator().listAnnotyzers();
			if (tyzersList != null && tyzersList.size() > 0) {
				for (Information i : tyzersList) {
					int noTargetTiers = 0;
					String idOrName = i.name;
					if (idOrName == null || idOrName.length() == 0) {
						// Log about hthe missing name
					} else {
						annotSelCB.addItem(idOrName);
						List<Parameter> parameters = i.parameters;
						if(parameters != null){
							for(int n=0; n < parameters.size(); n++){
								if(parameters.get(n).getValue() == TierNodeType.TARGET){
									noTargetTiers++;
								}
							}
						} 
						if(noTargetTiers == 0){
							noTargetTiers = 1;
						}
						
						if(noTargetTiers > maxTargetTiers){
							maxTargetTiers = noTargetTiers;
						}
						targetTiersMap.put(idOrName, noTargetTiers);
					}
				}			
			} else {
				//display a warning message...  no annotyzers are found
				JOptionPane.showMessageDialog(this, "No Annotyzer information found, cannot interlinearize.", "Warning", JOptionPane.WARNING_MESSAGE);
				//disable components
			}
		} 
        
		String[] columnIdentifiers = new String[maxTargetTiers+3];
		columnIdentifiers[0] = COLUMN_NO;
		columnIdentifiers[1] = "Annotyzer";
		columnIdentifiers[2] = "Source";
		for(int i= 1; i <= maxTargetTiers; i++){
			columnIdentifiers[2+i] = "Target"+i;
		}
        
        // DefaultTableModel
		configsModel = new DefaultTableModel(){
			@Override
			public boolean isCellEditable(int row, int column) {    
				if(getColumnName(column).equals(COLUMN_NO)){
	 				return false;
	 			}   
				
				Object value = getValueAt(row, column);
				
				if(value == null || value.equals(NO_TIERS)){
					return false;
				} else {
					return true;
				}				
			}
		};		
		configsModel.setColumnIdentifiers(columnIdentifiers);
		
		configsTable = new JTable(configsModel){
			@Override
			public void setEnabled(boolean enabled){
				super.setEnabled(enabled);
				if(enabled){
       	 			configsTable.setGridColor(Color.BLACK);  
       	 		} else {
       	 			configsTable.setGridColor(Color.GRAY); 
       	 		}		
			}
		};
		
		//DefaultTableCellRenderer
        DefaultTableCellRenderer render = new DefaultTableCellRenderer(){
        	private Font italic = new Font(configsTable.getFont().getFontName(), Font.ITALIC, configsTable.getFont().getSize());
        	private Font plain = new Font(configsTable.getFont().getFontName(), Font.PLAIN, configsTable.getFont().getSize());
       	 	
        	@Override
			public Component getTableCellRendererComponent(JTable table,
       			 Object value, boolean isSelected, boolean hasFocus, int row,
       			 	int column){
       		 
       	 		Component cell = super.getTableCellRendererComponent(table, value, 
       				 isSelected, hasFocus, row, column);       		 
       	 		
       	 		if(value == null){
       	 			if(column > SOURCE_INDEX){
       	 				String annotyzer = (String) table.getValueAt(row, ANNOTYZER_INDEX);
       	 				if(!annotyzer.equals(SELECT)){
       	 					int targetTiers = targetTiersMap.get(annotyzer);
       	 					if(SOURCE_INDEX+targetTiers < column){
       	 						cell.setBackground(Color.GRAY);
       	 						return cell;
       	 					} 
       	 				}
       	 			} 
       	 			cell.setBackground(Color.WHITE);
       	 			return cell;
       	 		}
       	 		
       	 		cell.setBackground(Color.WHITE);
       	 		
       	 		if(table.isEnabled()){
       	 			cell.setForeground(Color.BLACK);
       	 		} else {
       	 			cell.setForeground(Color.GRAY);
       	 		}				
       	 		       	 		
       	 		if(value.equals(SELECT)){    			   
       	 			cell.setFont(italic);       	 			
       	 		} else {
       	 			cell.setFont(plain);
       	 		}
       	 		return cell;    		   
       	 	}
        };        
        
        configsTable.setCellSelectionEnabled(true);	
        configsTable.setDefaultEditor(Object.class, new TableCellEditor());  
        configsTable.setDefaultRenderer(Object.class, render);      
        configsTable.setShowGrid(true);     
        configsTable.setSelectionBackground(Color.WHITE);        
        configsTable.setRowHeight(configsTable.getRowHeight()+ 5);  		
		configsTable.getColumnModel().getColumn(0).setMinWidth(50);       
		configsTable.getColumnModel().getColumn(0).setMaxWidth(50); 
		configsTable.setEnabled(false);
        
        Insets inset = new Insets(5, 10, 5, 10);      
        
        setLayout(new GridBagLayout());
		Insets insets = new Insets (4, 6, 4, 6);
		
		GridBagConstraints gbc;
		
		// config Panel
		JPanel sourceTypePanel = new JPanel(new GridBagLayout());
		sourceTypePanel.setBorder(new TitledBorder(ElanLocale.getString("InterlinearAnnotyzerConfigDlg.Mode")));
		gbc = new GridBagConstraints();
		gbc.insets = insets;
		gbc.gridx = 0;
		gbc.gridy = 0;		
		gbc.fill = GridBagConstraints.NONE;
		gbc.anchor = GridBagConstraints.NORTHWEST;	
		sourceTypePanel.add(tierRB, gbc);
		
		gbc.gridx = 1;			
		gbc.weightx = 1.0;		
		sourceTypePanel.add(typeRB, gbc);
		
		//button panel
        JPanel buttonPanel = new JPanel(new GridBagLayout());
        gbc = new GridBagConstraints();
	    gbc.gridx = 0;
	    gbc.gridy = 0;	   
		gbc.anchor = GridBagConstraints.NORTHWEST;	
	    gbc.insets = inset;
	    gbc.weightx = 1.0;
	    buttonPanel.add(removeButton, gbc);
	    
	    gbc.gridx = 2;	
	    gbc.weightx = 0.0;
	    gbc.anchor = GridBagConstraints.SOUTHEAST;
	    buttonPanel.add(addButton, gbc);
	    
	    gbc.gridx = 3;	
	    buttonPanel.add(cancelButton, gbc);
	    
		gbc = new GridBagConstraints();
		gbc.insets = insets;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 0.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;	
		getContentPane().add(sourceTypePanel, gbc);		
		
		gbc.gridy = 1;		
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weighty = 1.0;
		gbc.weightx = 1.0;
		getContentPane().add(new JScrollPane(configsTable), gbc);
		
		gbc.gridy = 2;
		gbc.weightx = 0.0;
		gbc.weighty = 0.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;	
		getContentPane().add(overwriteCB, gbc);
		
		gbc.gridy = 3;
		gbc.weightx = 0.0;
		gbc.fill = GridBagConstraints.HORIZONTAL;	
		getContentPane().add(buttonPanel, gbc);
	    
 	    addWindowListener(new WindowAdapter() {
 	    	@Override
			public void windowClosing(WindowEvent we) {
 	    		doClose();
            }
        }); 
 	    
 	    loadTierAndTypeMap(); 
		
		if(tiersMap.size() < 1){
			tierRB.setEnabled(false);
			typeRB.setEnabled(false);
			// display a warning message and close the dialog...			
			JOptionPane.showMessageDialog(this, 
					ElanLocale.getString("InterlinearAnnotyzerConfigDlg.NoTier"), 
					"Warning", JOptionPane.WARNING_MESSAGE);
			//disable components
		} else {
			List<AnnotConfig> list = 
					((InterlinearMediator)manager.getMediator()).getConfigurations();
			List<Object> mappings;
			for(int i = 0; i < list.size(); i++){
				mappings = new ArrayList<Object>();
				mappings.add(list.get(i).annotId.name);
				mappings.add(list.get(i).getSource());
				mappings.add(list.get(i).getDest());
				
				if(list.get(i).isTypeMode()){
					typeConfigMap.put(typeConfigMap.size()+1, mappings);
				} else {
					tierConfigMap.put(tierConfigMap.size()+1, mappings);
				}
			}
			readPreferences();
		}
    }
    
    /**
     * Stores the user preferences
     */
    private void storePreferences(){
    	Preferences.set("AnnotationConfigurationDialog.TypeMode", typeRB.isSelected(), manager.getTranscription());
    }
    
    /**
     * Reads and loads the user preferences
     */
    private void readPreferences(){
    	Boolean boolPref = Preferences.getBool("AnnotationConfigurationDialog.TypeMode", manager.getTranscription());
    	if (boolPref != null && boolPref.booleanValue()) {
    		typeRB.setSelected(true);
            updateTableForMode(TYPE_MODE,false);
        } else {
           	tierRB.setSelected(true);
           	updateTableForMode(TIER_MODE, false);
    	}
    }
    
    @Override
	public void actionPerformed(ActionEvent e) {
    	Object source = e.getSource();    	
    	if(source == tierRB){
    		if(currentMode != TIER_MODE){
    			updateTableForMode(TIER_MODE, true);
    		}    		
    	} else if(source == typeRB){
    		if(currentMode != TYPE_MODE){
    			addConfigurations();
    			updateTableForMode(TYPE_MODE, true);
    		}
    	} else if(source == cancelButton){
    		tierConfigMap.clear();
    		typeConfigMap.clear();
    		storePreferences();
    		doClose();
    	} else if(source == addButton){
    		addConfigurations();
    		storePreferences();
    		doClose();    		
    	} else if(source == removeButton){
    		int row = configsTable.getSelectedRow();
    		int rowcount = configsTable.getRowCount();   
    		
    		Object obj[] = new Object[maxTargetTiers+3];
    		obj[0] = 0;
    		obj[1] = SELECT;
    		obj[2] = null;
    		for(int i=1; i <= maxTargetTiers; i++){
    			obj[2+i] = null;
    		}
    		
    		if(row == (rowcount-1)){
    			configsModel.removeRow(row);
    			configsModel.addRow(obj);
    			configsModel.setValueAt(configsTable.getRowCount(), configsTable.getRowCount()-1, 0);
    		} else if (row > -1) {
				int n = row+1;
				while(n < rowcount){
					configsModel.setValueAt(n, n, 0);
					n++;
				}				
				configsModel.removeRow(row);
				
				if(configsModel.getRowCount() == 0){
					configsModel.addRow(obj);
	    			configsModel.setValueAt(1, 0, 0);
	    		} else {
					String t = (String) configsModel.getValueAt(configsModel.getRowCount()-1, 1);
					if( t == null || t.equals(NO_TIERS)){
						configsModel.setValueAt(SELECT, configsModel.getRowCount()-1, 1);
					} else if (!t.equals(SELECT)){
						configsModel.addRow(obj);
		    			configsModel.setValueAt(configsTable.getRowCount(), configsTable.getRowCount()-1, 0);
					}
				}
				((TableCellEditor)configsTable.getDefaultEditor(Object.class)).setLastUpdatedRow(-1);
			}
    	}
    }
    
    /**
	 * Loads all possible source and destinations that can be mapped together
	 * to create a configuration in a map
	 */
	private void loadTierAndTypeMap(){          
	    List<TierImpl> list = ((TranscriptionImpl)manager.getTranscription()).getTiers();
	    List<TierImpl> childList;
	  	TierImpl tier;	
	  	List<String> childTiersList;
	  	List<String> childTypesList = null;
	  	String typeName;
	  		
	  	for (int i = 0; i < list.size(); i++) {      		
	  		tier = list.get(i);  
	  		typeName = tier.getLinguisticType().getLinguisticTypeName();
	  		childTypesList = null;
	  		if(typesMap.containsKey(typeName)){
	  			childTypesList = typesMap.get(typeName);
	  		} 
	  		
	  		childList = tier.getChildTiers();
	  		if(childList != null && childList.size() > 0){
	  			childTiersList = new ArrayList<String>();
	  			if(childTypesList == null){
	  				childTypesList = new ArrayList<String>();
	  			}
	  			for (TierImpl child : childList) { 
	  				childTiersList.add(child.getName());	
	  				if(!childTypesList.contains(child.getLinguisticType().getLinguisticTypeName())){
	  					childTypesList.add(child.getLinguisticType().getLinguisticTypeName());
	  				}
	  				//addChildren((TierImpl)child, childTiersList, childTypesList);
	  			}        			
	  			
	  			tiersMap.put(tier.getName(), childTiersList);
	  			typesMap.put(typeName, childTypesList);      			
	  		}
	  	}
	}
	/**
	 * HS 25 Sept: Is this sufficient?
	 * @param t
	 * @param childTiersList
	 * @param childTypesList
	 */
	private void addChildren(TierImpl t, List<String> childTiersList, List<String> childTypesList) {
		List<TierImpl> children = t.getChildTiers();
		if (children != null && children.size() > 0) {
			for (TierImpl child : children) { 
  				childTiersList.add(child.getName());	
  				if(!childTypesList.contains(child.getLinguisticType().getLinguisticTypeName())){
  					childTypesList.add(child.getLinguisticType().getLinguisticTypeName());
  				}
			}
		}
	}

	/**
     * Add the configurations to a map
     */
    private void addConfigurations(){
		String sourceName;
		List<String> destList;
		String destName;
		String tyzerName;
		List<Object> mappingList;
		Map<Integer, List<Object>> configMap;
		
		if(currentMode == TIER_MODE){
			tierConfigMap.clear();
			configMap = tierConfigMap;
		} else {
			typeConfigMap.clear();
			configMap = typeConfigMap;
		}
		
		for(int i=0; i < configsTable.getRowCount(); i++ ){
			destList = new ArrayList<String>();
			tyzerName = (String) configsTable.getValueAt(i, 1);
			if(!tyzerName.equals(SELECT)){
				sourceName = (String) configsTable.getValueAt(i, 2);
				if(!sourceName.equals(NO_TIERS) && !sourceName.equals(SELECT)){
					for( int c= 3; c < configsTable.getColumnCount(); c++){
						destName =(String) configsTable.getValueAt(i, c);
						if(destName != null && !destName.equals(SELECT)){
							destList.add(destName);
						} 
					}
					
					if(targetTiersMap.get(tyzerName) == destList.size()){
						mappingList = new ArrayList<Object>();
						mappingList.add(tyzerName);
						mappingList.add(sourceName);
						mappingList.add(destList);						
						
						configMap.put(configMap.size()+1, mappingList);
					}
				}
			}
		}
    }    
    
    /**
     * Returns the configurations map
     * 
     * @return HashMap<Integer, List<String>>, a map with all configurations
     * 
     * Details regarding the contents of the map
     * Integer: integer starts with 1 as a index
     * List<Object> contains 3 values : [0]<string> - annotyzer, [1]<List<String> -destination, [2]<String> - source
     */
    public Map<Integer, List<Object>> getConfigurationMap(){
    	if(currentMode == TIER_MODE){
			return tierConfigMap;
		} else {
			return typeConfigMap;
		}
    }
    
    /**
     * Check if the configuration is based on type mapping
     * 
     * @return true, if type based
     *               else false (tier based)
     */
    public boolean isTypeMode(){
    	if(currentMode == TIER_MODE){
			return false;
		} else {
			return true;
		}
    }
    
    /**
     * Checks whether to overwrite the existing configuration
     * 
     * @return true - overwrites, else no
     */
    public boolean isOverwrite(){    	
    	return overwriteCB.isSelected();
    }
    
    /**
     * Updates the table when the mode is changed from/to tier/type mode
     * 
     * @param mode, (TIER_MODE / TYPE_MODE) new mode to be updated
     */
    private void updateTableForMode(int mode, boolean saveCurrentMapping){   
    	if(configsTable.isEditing()){
    		configsTable.editingStopped(new ChangeEvent(this));	
    	}
    	
    	if(saveCurrentMapping){
    		addConfigurations();
    	}
    	
    	while(configsTable.getRowCount() > 0){
    			configsModel.removeRow(configsTable.getRowCount()-1);
    	}
			
		//update the table for the new mode	
		List<Object> mappingList;
		Map<Integer, List<Object>> configMap;
		
		if(mode == TIER_MODE){
			configMap = tierConfigMap;
		} else {
			configMap = typeConfigMap;
		}
		
		int n = 1;
		Object[] rowEntry;
		List<String> targetList;
		while(n <= configMap.size()){
			mappingList = configMap.get(n);
			rowEntry = new Object[maxTargetTiers+3];
			
			rowEntry[0] = n;
			rowEntry[1] = mappingList.get(0);
			rowEntry[2] = mappingList.get(1);
			
			targetList = (List<String>)mappingList.get(2);
    		for(int i=1; i <= maxTargetTiers; i++){
    			if(i <= targetList.size()){
    				rowEntry[2+i] = targetList.get(i-1);
    			} else {
    				rowEntry[2+i] = null;
    			}
    		}
			
			configsModel.addRow(rowEntry);	
			n++;
		}
			
		if(configMap.size() > 0){
			configsModel.addRow(new Object[]{ configMap.size()+1 , SELECT, null, null});
		} else {				
			configsModel.addRow(new Object[]{ 1, SELECT, null, null});
		}
		
		configsTable.setEnabled(true);
		currentMode = mode; 
    }
	
	/**
     * Closes this dialog
     */
    private void doClose() {
		setVisible(false);
		dispose();
	} 
    
    /**
     * Loads the possible values for the given row
     * in a map, depending on the previous mappings made 
     * 
     * {@value} currentRowMap : contains the possible mappings for the 
     * given row
     */
    private void loadValuesRow(int row){
    	currentRowMap.clear();
    	Iterator it;
    	if(tierRB.isSelected()){
    		it = tiersMap.entrySet().iterator();
    	} else {
    		it = typesMap.entrySet().iterator();
    	}
    	
    	while(it.hasNext()){
    		Entry<String, List<String>> e = (Entry<String, List<String>>) it.next();
    		if(e.getValue().size() >= targetTiersMap.get(configsTable.getValueAt(row, ANNOTYZER_INDEX))){
    			currentRowMap.put(e.getKey(), e.getValue());
			} 
    	}    	
    	
    	String sourceName ;
    	String destName;
    	List<String> destList;
    	
    	for(int i=0; i < configsTable.getRowCount(); i++){    		
    		if(i == row){
    			continue;
    		}
    		sourceName = (String) configsTable.getValueAt(i, 2);
    		
    		if(!currentRowMap.containsKey(sourceName)){
    			continue;
    		}
    		
    		destList = new ArrayList<String>();
			destList.addAll(currentRowMap.get(sourceName));
    		
			if(!sourceName.equals(NO_TIERS) && !sourceName.equals(SELECT)){
    			for( int c= SOURCE_INDEX+1; c < configsTable.getColumnCount(); c++){
					destName =(String) configsTable.getValueAt(i, c);
					if(destName != null && !destName.equals(SELECT)){
						if(destList.contains(destName)){
		    				destList.remove(destName);
		    			}
					} 
				} 			
    			
    			if(destList.size() < targetTiersMap.get(configsTable.getValueAt(row, ANNOTYZER_INDEX))){
    				currentRowMap.remove(sourceName);
    			} else {
    				currentRowMap.put(sourceName, destList);
    			}
    		}
    	}
    }
	
	/**
     * Cell Editor for the selection JTable
     * 
     * @author aarsom
     *
     */
    private class TableCellEditor extends DefaultCellEditor implements ActionListener {    	
    	private int startEditInOneClick = 1;
    	private int column;  
    	private int row; 
    	private int lastUpdatedRow = -1;

    	public TableCellEditor() {
    		super(new JComboBox());
    		setClickCountToStart(startEditInOneClick);      		
    	}
    	
    	public void setLastUpdatedRow(int row){
    		lastUpdatedRow = row;
    	}
    	
    	@Override
		public Component getTableCellEditorComponent(
    			JTable table,
    			Object value,
    			boolean isSelected,
    			int row,
    			int column) { 
    		
    		this.column = column;
    		this.row = row;
    		
    		if(column == ANNOTYZER_INDEX){
    			annotSelCB.removeActionListener(this);   
    			if( value.equals(SELECT)){
    				annotSelCB.setSelectedItem(null);
    			} else {
    				annotSelCB.setSelectedItem(value);    				
    			}
    			annotSelCB.addActionListener(this);    			
    			return annotSelCB;
    		} 
    		
    		if(lastUpdatedRow != row){
    			loadValuesRow(row);
    			lastUpdatedRow = row;
    		}
    		
    		if(column == SOURCE_INDEX){    			
    			sourceCB.removeActionListener(this);
    			sourceCB.removeAllItems();
    			
    			if(currentRowMap.size() < 1){
    				table.setValueAt(NO_TIERS, row, column);
    				return null;
    			}
    			
    			Iterator it = currentRowMap.keySet().iterator();
    			while(it.hasNext()){
    				sourceCB.addItem(it.next());
    			}
    			
    			if( value.equals(SELECT) ){
    				sourceCB.setSelectedItem(null);
    			} else {
    				sourceCB.setSelectedItem(value);    				
    			}
    			sourceCB.addActionListener(this);
    			return sourceCB;
			}
    		
    		destinationCB.removeActionListener(this);    	
    		destinationCB.removeAllItems();
    		
    		List<String> destList = new ArrayList<String>();
    		destList.addAll(currentRowMap.get(table.getValueAt(row, SOURCE_INDEX)));
    		for( int c= SOURCE_INDEX+1; c < column; c++){
				String destName =(String) configsTable.getValueAt(row, c);
				if(destList.contains(destName)){
					destList.remove(destName);
	    		}
			} 
    		
    		// null pointer exeception
    		for(String n : destList){
    			destinationCB.addItem(n);
    		}
    		
    		if( value.equals(SELECT)){
    			destinationCB.setSelectedItem(null);
    		} else {
    			destinationCB.setSelectedItem(value);    				
    		}
    		destinationCB.addActionListener(this);
    		return destinationCB;
    	}
    	
    	@Override
		public Object getCellEditorValue() {      		
    		Object editorValue = null;
    		if(column == ANNOTYZER_INDEX){
    			editorValue =  annotSelCB.getSelectedItem();
    		} else if(column == SOURCE_INDEX){
    			editorValue =  sourceCB.getSelectedItem();
    		}else {
    			editorValue = destinationCB.getSelectedItem();   
    		}
    		
    		if(editorValue == null){
    			return SELECT;
    		}    	
    		
    		return editorValue;   		
    	}

		@Override
		public void actionPerformed(ActionEvent e) {
			if(!(e.getSource() instanceof JComboBox)){
				return;
			}
			
			Object selectedItem = getCellEditorValue() ;
			
			if(column == ANNOTYZER_INDEX){
				lastUpdatedRow = -1;
			}
			
			if(selectedItem.toString().equals(configsTable.getValueAt(row, column))){
				configsTable.editingStopped(new ChangeEvent(this));	
				return ;
			}
			
			configsTable.editingStopped(new ChangeEvent(this));	
			
			int nextColumn = column+1;
			
			if (selectedItem.toString().equals(SELECT)){	
				while (nextColumn <= configsTable.getColumnCount()-1){
					configsTable.setValueAt(null, row, nextColumn);
					nextColumn++;
				}
				return;		
			}
			
			// if last row, then add new row
			if((column == targetTiersMap.get(configsTable.getValueAt(row, ANNOTYZER_INDEX))+SOURCE_INDEX ||
					column == (configsTable.getColumnCount()-1)) && 
					row == (configsTable.getRowCount() -1)){	
				Object obj[] = new Object[maxTargetTiers+3];
				obj[0] = configsTable.getRowCount()+1;
				obj[1] = SELECT;
				obj[2] = null;
	    		for(int i=1; i <= maxTargetTiers; i++){
	    			obj[2+i] = null;
	    		}
	    		configsModel.addRow(obj);
	    		return;
			}
			
			if(column == targetTiersMap.get(configsTable.getValueAt(row, ANNOTYZER_INDEX))+SOURCE_INDEX){
				return;
			}
			
			while (nextColumn <= configsTable.getColumnCount()-1){
				if(nextColumn == column+1){
					configsTable.setValueAt(SELECT, row, nextColumn);
				} else {
					configsTable.setValueAt(null, row, nextColumn);
				}
				nextColumn++;
			}
		}
    }
}
