package mpi.eudico.client.annotator.interlinear.edit;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.layout.InterlinearizationManager;
import mpi.eudico.server.corpora.clom.Tier;
import nl.mpi.lexan.annotyzers.helpers.Information;

/** 
 * A class for selecting and configuring available "annotyzers".
 * 
 * @author Han Sloetjes
 * @author Aarthy Somasundaram
 *
 */
@SuppressWarnings("serial")
public class AnnotConfigPanel extends JPanel implements ComponentListener, 
	ActionListener, ListSelectionListener {
	private InterlinearizationManager manager;	
	private JButton configButton;	
	private JButton removeConfigButton;
	private JCheckBox showTiersCB;
	private JTable configsTable;
	private DefaultTableModel configsModel;
	private JPanel configPanel;
	
	public AnnotConfigPanel(InterlinearizationManager manager) {
		super();
		this.manager = manager;
		initComponents();
	}
	
	private void initComponents() {
		setLayout(new GridBagLayout());
		Insets insets = new Insets (4, 6, 4, 6);
		
		GridBagConstraints gbc = new GridBagConstraints();
		
		// config Panel
		configPanel = new JPanel(new GridBagLayout());
		configPanel.setBorder(new TitledBorder(""));   
       
		gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.insets = insets;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		//add(scrollPane, gbc);
		
		add(configPanel, gbc);
		// table 
		configsModel = new DefaultTableModel(new String[]{"Annotyzer", "Source", "Target", "LinType"}, 0){
			@Override
			public boolean isCellEditable(int row, int column) {	
				 return false;
			 }
		};
		
		configsTable = new JTable(configsModel);
		
		//DefaultTableCellRenderer
        DefaultTableCellRenderer render = new DefaultTableCellRenderer(){
        	private Font italic = new Font(configsTable.getFont().getFontName(), Font.ITALIC, configsTable.getFont().getSize()-4);
        	private Font plain = new Font(configsTable.getFont().getFontName(), Font.PLAIN, configsTable.getFont().getSize());
       	 	
        	@Override
			public Component getTableCellRendererComponent(JTable table,
       			 Object value, boolean isSelected, boolean hasFocus, int row,
       			 	int column){
       		 
       	 		Component cell = super.getTableCellRendererComponent(table, value, 
       				 isSelected, hasFocus, row, column);  
       	 		
       	 		if( value != null && table.getValueAt(row, 0) == null){
       	 			cell.setFont(italic);  
       	 			cell.setBackground(Color.LIGHT_GRAY);
       	 			cell.setForeground(Color.WHITE);
       	 		} else {
       	 			cell.setFont(plain);
       	 			cell.setBackground(Color.WHITE);
       	 			cell.setForeground(Color.BLACK);
	 			
       	 			// tier
       	 			if(value != null && !(Boolean)configsModel.getValueAt(row,3)){
       	 				cell.setBackground(Color.LIGHT_GRAY);
       	 				cell.setForeground(Color.WHITE);
       	 			} 
       	 		} 		
       	 		       	 		
       	 		if(table.getSelectedRow() == row){       	 			  	 			
       	 			cell.setBackground(table.getSelectionBackground());
       	 			cell.setForeground(table.getSelectionForeground());
       	 		}
       	 		
       	 		return cell;    		   
       	 	}
        };  
       
		configsTable.setDefaultRenderer(Object.class, render);
		configsTable.getTableHeader().setReorderingAllowed(false);
		configsTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		configsTable.getSelectionModel().addListSelectionListener(this);
		configsTable.removeColumn(configsTable.getColumn("LinType"));
		
		removeConfigButton = new JButton();
		removeConfigButton.addActionListener(this);
		
		configButton = new JButton();
		configButton.addActionListener(this);
		
		showTiersCB = new JCheckBox();
		showTiersCB.setSelected(true);
		showTiersCB.addActionListener(this);
		
		gbc = new GridBagConstraints();
		gbc.insets = insets;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.gridwidth = 2;
		configPanel.add(new JScrollPane(configsTable), gbc);
		
		gbc.gridx = 0;
		gbc.gridy = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 0.0;
		gbc.fill = GridBagConstraints.NONE;
		gbc.anchor = GridBagConstraints.NORTHWEST;
		configPanel.add(showTiersCB, gbc);	
		
		gbc.gridwidth = 1;		
		gbc.gridy = 2;
		gbc.anchor = GridBagConstraints.SOUTHEAST;
		configPanel.add(removeConfigButton, gbc);
		
		gbc.gridx = 1;	
		gbc.anchor = GridBagConstraints.NORTHWEST;
		configPanel.add(configButton, gbc);			
		
		addComponentListener(this);
		
		updateLocale();
	}
	
	public void updateLocale(){
		((TitledBorder)configPanel.getBorder()).setTitle(ElanLocale.getString("InterlinearAnnotyzerConfigDlg.Title"));
		removeConfigButton.setText(ElanLocale.getString("FileChooser.Button.Remove"));
		configButton.setText(ElanLocale.getString("InterlinearAnnotyzerConfigPanel.AddConfig"));
		showTiersCB.setText(ElanLocale.getString("InterlinearAnnotyzerConfigPanel.ShowTiers"));
	}
	
	/**
	 * Changes from the preferences
	 */
	public void configsChanged() {
		configsTable.getSelectionModel().removeListSelectionListener(this);
		
		if (configsModel.getRowCount() > 0) {
			for (int i = configsModel.getRowCount() - 1; i >= 0; i--) {
				configsModel.removeRow(i);
			}
		}
		
		List<AnnotConfig> conf = ((InterlinearMediator) manager.getMediator()).getConfigurations();
				
		AnnotConfig activeConf = manager.getInterEditor().getActiveConfiguration();
		int activeConfRowIndex = -1;
		
		String target;
		for (AnnotConfig ac : conf) {
			target = null;
			for(String s : ac.getDest()){
				if(target == null){
					target = s;
				} else {
					target = target.concat(",").concat(s);
				}
			}
			configsModel.addRow(new Object[]{ac.getAnnotId().name, ac.getSource(), target, ac.isTypeMode()});
			
			if(activeConf == null){
				activeConf = ac;
				activeConfRowIndex = 0;
				manager.getInterEditor().setActiveConfiguration(ac);
			}
			
			if(ac.isTypeMode() && showTiersCB.isSelected()){
				List<AnnotConfig> tierConfig = ((AnnotTypeConfig)ac).getTierConfigurations();
				
				for (AnnotConfig atc : tierConfig) {	
					target = null;
					for(String s : atc.getDest()){
						if(target == null){
							target = s;
						} else {
							target = target.concat(",").concat(s);
						}
					}
						configsModel.addRow(new Object[]{null, atc.getSource(), target, null});	
						
						if(activeConf != null && activeConfRowIndex < 0){
							if(activeConf.equals(atc)){									
								activeConfRowIndex = configsModel.getRowCount()-1;
							}
						}
					}
				}
			
				if(activeConf != null && activeConfRowIndex < 0){
					if(activeConf.equals(ac)){									
						activeConfRowIndex = configsModel.getRowCount()-1;
					}
				}
			}
		
		configsTable.getSelectionModel().addListSelectionListener(this);
		
		if(activeConfRowIndex < 0 && conf.size() > 0){
			configsTable.getSelectionModel().setSelectionInterval(configsTable.getRowCount()-1, configsTable.getRowCount()-1);
			manager.getInterEditor().setActiveConfiguration(conf.get(0));
		} else {
			configsTable.getSelectionModel().setSelectionInterval(activeConfRowIndex, activeConfRowIndex);
		}
	}
	
	/**
	 * Updates the configuration table with the configurations
	 * from the config dialog
	 *  
	 * @param configMap : <no, List<annotyzerName (String) , source(String), targetList(List<String>)>
	 * @param typeMode : true if the mapping is based on linguistic types
	 * @param overwrite, if true overwrite existing configs
	 */
	public void addConfigurations(Map<Integer, List<Object>> configMap, 
			boolean typeMode, boolean overwrite) {
		
		if(configMap == null || configMap.size() == 0){
			return;
		}
		
		String source;
		List<String> targetList;
		String annot;
		
		AnnotConfig ac;
		AnnotTypeConfig atc;
		Information info = null;			
		
		List<Object> annotConfigList;
		List<Information> tyzers = manager.getMediator().listAnnotyzers();
		
		int n = 1;
		while(n <= configMap.size()){
			annotConfigList = configMap.get(n);
			annot = (String) annotConfigList.get(0);
			source = (String) annotConfigList.get(1);
			targetList = (List<String>) annotConfigList.get(2);
			
			// find the annotyzer			
			if (tyzers != null && tyzers.size() > 0) {				
				for (Information li : tyzers) {
					if (annot.equals(li.name)) {
						info = li;
						break;
					}
				}
			}
			
			// if info == null, popup warning
			if (info == null) {
				JOptionPane.showMessageDialog(this, annot + ": "+ 
			ElanLocale.getString("InterlinearAnnotyzerConfigPanel.AnnotyzerNotFound")
						+" -" + source,	"Warning", JOptionPane.WARNING_MESSAGE);
			} else {	
				if(typeMode){					
					atc = new AnnotTypeConfig(info, source, targetList);		
					
					if(overwrite){
						overwriteConf(atc);
					} else if (srcConfigExists(atc)){
						n++;
						continue;
					}
					
					List<? extends Tier> sourceTiers = manager.getTranscription().getTiersWithLinguisticType(source);
					HashMap<String, List<String>> configTierMap = new HashMap<String, List<String>>();
					String dest;
					List<String> targetTierList;
					List<? extends Tier> childTierList;
					
					for(int l= 0; l< targetList.size(); l++){	
						List<? extends Tier> destTiers = manager.getTranscription().
								getTiersWithLinguisticType(targetList.get(l));
						
						for(int i= 0; i< sourceTiers.size(); i++){	
							childTierList = sourceTiers.get(i).getChildTiers();
							
							for(int t= 0; t < childTierList.size(); t++){
								if(destTiers.contains(childTierList.get(t))){
									dest = childTierList.get(t).getName();
									source = sourceTiers.get(i).getName();	
									
									if(configTierMap.containsKey(source)){
										targetTierList = configTierMap.get(source);
									} else {
										targetTierList = new ArrayList<String>();
									}
									
									if(!targetTierList.contains(dest)){
										targetTierList.add(dest);
										configTierMap.put(source, targetTierList);
										break;
									}
								}
							}
						}
					}
					
					Iterator it = configTierMap.entrySet().iterator();			    	
			    	while(it.hasNext()){
			    		Entry<String, List<String>> e = (Entry<String, List<String>>) it.next();
			    		if(e.getValue().size() == targetList.size()){
			    			ac = new AnnotConfig(info, e.getKey(), e.getValue());
			    			
			    			if(overwrite){
								overwriteConf(ac);
							} else if (srcConfigExists(ac)){
								continue;
							}
							atc.addTierConf(ac);
						} 
			    	}
			    	
			    	if(atc.getTierConfigurations().size() > 0){
						((InterlinearMediator) manager.getMediator()).addConfig(atc);
					} else {
						// show warnining
					}
				} else {
					ac = new AnnotConfig(info, source, targetList);
					
					if(overwrite){
						overwriteConf(ac);
					} else if (srcConfigExists(ac)){
						n++;
						continue;
					}
					
					((InterlinearMediator) manager.getMediator()).addConfig(ac);
				}
			}
			n++;
		}
		
		configsChanged();
	}
	
	/**
	 * 
	 * @param ac
	 */
	private void overwriteConf(AnnotConfig ac){
		List<AnnotConfig> confList = ((InterlinearMediator) manager.getMediator()).getConfigurations();
		if(ac instanceof AnnotTypeConfig){
			for(AnnotConfig atc : confList){
				if(atc.equals(ac)){
					((InterlinearMediator) manager.getMediator()).removeConfig(atc);
					break;
				}
			}
		} else{			
			for(AnnotConfig atc : confList){
				if(atc instanceof AnnotTypeConfig){					
					for(AnnotConfig tc : ((AnnotTypeConfig)atc).getTierConfigurations()){
						if(tc.equals(ac)){
							((AnnotTypeConfig) atc).removeTierConf(tc);
							break;
						}
					}
					if(((AnnotTypeConfig)atc).getTierConfigurations().size() == 0){
						((InterlinearMediator) manager.getMediator()).removeConfig(atc);
					}
				} else if(atc.equals(ac)){
					((InterlinearMediator) manager.getMediator()).removeConfig(atc);
					break;
				}
			}
		}
	}
	
	/**
	 * Check if the given config already exists
	 * 
	 * @param ac, config to check is already exists
	 * @return true if yes, else false
	 */
	private boolean srcConfigExists(AnnotConfig ac){
		List<AnnotConfig> confList = ((InterlinearMediator) manager.getMediator()).getConfigurations();
		if(ac instanceof AnnotTypeConfig){
			for(AnnotConfig atc : confList){
				if(atc.equals(ac)){
					return true;
				}
			}
		} else{			
			for(AnnotConfig atc : confList){
				if(atc instanceof AnnotTypeConfig){					
					for(AnnotConfig tc : ((AnnotTypeConfig)atc).getTierConfigurations()){
						if(tc.equals(ac)){
							return true;
						}
					}
				} else if(atc.equals(ac)){
					return true;
				}
			}
		}		
		return false;
	}
	
	/**
	 * Get the targetList from the given string
	 * 
	 * @param target, string in which the targets are comma seperated 
	 * @return targetList
	 */
	public List<String> getTargetList(String target){
		List<String> targetList = null;
		if(target != null){
			targetList = new ArrayList<String>();
			String[] targets = target.split(",");
			for(String s : targets){
				targetList.add(s);
			}
		}
		return targetList;
		
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == removeConfigButton) {
			int row = configsTable.getSelectedRow();
			if (row > -1) {
				String at = (String) configsModel.getValueAt(row, 0);
				String st = (String) configsModel.getValueAt(row, 1);
				List<String> dt = getTargetList((String) configsModel.getValueAt(row, 2));
				Boolean mode = (Boolean) configsModel.getValueAt(row, 3);
				
				List<AnnotConfig> conf = ((InterlinearMediator) manager.getMediator()).getConfigurations();
				
				if(at == null){
					for (AnnotConfig atc : conf) {
						if(atc instanceof AnnotTypeConfig){
							for(AnnotConfig ac : ((AnnotTypeConfig)atc).getTierConfigurations()){
								if (st.equals(ac.getSource()) &&
									dt.equals(ac.getDest()) &&
									!ac.isTypeMode()) {
									
									((AnnotTypeConfig) atc).removeTierConf(ac);
									((InterlinearMediator) manager.getMediator()).removeConfig(ac);
									configsModel.removeRow(row);
									
									if(((AnnotTypeConfig)atc).getTierConfigurations().size() == 0){
										configsModel.removeRow(row-1);
										((InterlinearMediator) manager.getMediator()).removeConfig(atc);
									}
									break;
								}
							}
						}
					}					
				} else {
					for (AnnotConfig ac : conf) {
						if (at.equals(ac.getAnnotId().name) &&
								st.equals(ac.getSource()) &&
								dt.equals(ac.getDest()) &&
								mode == ac.isTypeMode()) {
							
							if(ac instanceof AnnotTypeConfig && showTiersCB.isSelected()){
								//remove all rows
								int n = row+1;
								while(configsModel.getValueAt(n, 0) == null){
									configsModel.removeRow(n);
									
									if(configsModel.getRowCount() <= n){
										break;
									}
								}
							}
							
							configsModel.removeRow(row);
							
							((InterlinearMediator) manager.getMediator()).removeConfig(ac);
							break;
						}
					}
				}
				
				repaint();
			}
		}		
		else if(e.getSource() == configButton){
			AnnotConfigDialog configDialog = new AnnotConfigDialog(null, manager);
			configDialog.setVisible(true);
			addConfigurations(configDialog.getConfigurationMap(), configDialog.isTypeMode(), configDialog.isOverwrite());
		} else if(e.getSource() == showTiersCB){
			configsChanged();
		}
	}


	@Override
	public void componentHidden(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void componentMoved(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void componentResized(ComponentEvent e) {
		revalidate();
	}

	@Override
	public void componentShown(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void valueChanged(ListSelectionEvent e) {
		int row = configsTable.getSelectedRow();
		if (row > -1) {
			String at = (String) configsModel.getValueAt(row, 0);
			String st = (String) configsModel.getValueAt(row, 1);
			List<String> dt = getTargetList((String) configsModel.getValueAt(row, 2));
			
			Boolean mode = (Boolean) configsModel.getValueAt(row, 3);
			
			List<AnnotConfig> conf = ((InterlinearMediator) manager.getMediator()).getConfigurations();
			
			if(at == null){
				for (AnnotConfig atc : conf) {
					if(atc instanceof AnnotTypeConfig){
						for (AnnotConfig ac : ((AnnotTypeConfig)atc).getTierConfigurations()) {
							if ( st.equals(ac.getSource()) &&
									dt.equals(ac.getDest()) && 
									!ac.isTypeMode()){
								manager.getInterEditor().setActiveConfiguration(ac);
								break;
							}
						}
					}
				}
			} else {
				for (AnnotConfig ac : conf) {
					if ( at.equals(ac.getAnnotId().name) && 
							st.equals(ac.getSource()) &&
							dt.equals(ac.getDest()) &&
							mode == ac.isTypeMode()){
						manager.getInterEditor().setActiveConfiguration(ac);
						break;
					}
				}
			}
		}		
	}
}
