package mpi.eudico.client.annotator.interlinear.edit;

import java.awt.Color;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.List;

import mpi.eudico.client.annotator.interlinear.IGTTierType;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTAnnotation;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTDataModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTTier;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTNodeRenderInfo;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTViewerRenderInfo;

/**
 *  A class for rendering a row of data (a group) into the specified graphics context.
 *  
 * @author Han Sloetjes
 */
public class IGTGroupRenderer {
	private boolean debug = true;
	private final Color DEBUG_BORDER_COLOR = Color.LIGHT_GRAY;
	//private final Color BORDER_COLOR = Color.DARK_GRAY;
	/**
	 * 
	 * @param model
	 * @param g2d
	 * @param yTranslation
	 * @param selected
	 */
//	public void renderIGTGroup(IGTDataModel model, Graphics2D g2d, int yTranslation, boolean selected) {
//		
//	}
	
	/**
	 * Renders a row / group into the specified graphics context.
	 * 
	 * 
	 */
	public void renderRow(Graphics2D g2d, IGTDataModel model, IGTViewerRenderInfo vrInfo, int row,
			int xOff, int yOff, int width, int height) {
		// clear the rectangle for this row
		if (row % 2 == 0) {
			g2d.setColor(vrInfo.backgroundColor);
		} else {
			g2d.setColor(vrInfo.backgroundColor2);
		}
		g2d.fillRect(xOff, yOff, width, height);
		
		g2d.setColor(vrInfo.foregroundColor);
		g2d.drawLine(vrInfo.headerWidth, yOff, vrInfo.headerWidth, yOff + height);
		
		// start rendering tiers, tier labels, annotations
		int tempY = yOff;
		IGTTier tier;
		IGTAnnotation igtAnn;
		IGTNodeRenderInfo annRender;
		
		List<IGTTier> renderedTier = new ArrayList<IGTTier>();
		for (int i = 0; i < model.getRowCount(); i++) {
			tier = model.getRowData(i);
			int tierHeight = vrInfo.getHeightForTier(g2d, tier.getTierName());
			if (renderedTier.contains(tier)) {
				continue;
			}
			
			if (tier.getType() == IGTTierType.WORD_LEVEL_ROOT /*|| tier.getType() == IGTTierType.SUBDIVISION*/) {
				int numBlocks = 1;
				List<Integer> wrapIndices = tier.getRenderInfo().getWrapIndices();
				if (wrapIndices != null) {
					numBlocks = wrapIndices.size() + 1;
				}
				// render the group of tiers in blocks with or without repeating the tier label
				IGTTier depTier;
				for (int j = 0; j < numBlocks; j++) {
					tempY += tierHeight; 
					g2d.drawString(model.getShortTierNameForIndex(i), xOff, tempY - 2); // position the baseline of the text correctly
					// render annotations on this tier 
					
					int minA = 0;
					int maxA = tier.getAnnotations().size() - 1;
					if (maxA < 0) {// no annotations
						continue; 
					}
					
					if (j > 0) {
						minA = wrapIndices.get(j - 1);
					} 
					if (j < numBlocks - 1) {
						maxA = wrapIndices.get(j) - 1;
					}
		
					
					for (int a = minA; a <= maxA; a++) {
						igtAnn = tier.getAnnotations().get(a);
						// hier render annotation
						g2d.drawString(igtAnn.getTextValue(), xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, 
								tempY - 2);// TODO position the text correctly vertically
						if (debug) {
							g2d.setColor(DEBUG_BORDER_COLOR);
							g2d.drawRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - tierHeight, 
									igtAnn.getRenderInfo().calcWidth, tierHeight);
							g2d.setColor(vrInfo.foregroundColor);
						}
					}
					int chMinX = tier.getAnnotations().get(minA).getRenderInfo().x;
					annRender = tier.getAnnotations().get(maxA).getRenderInfo();
					int chMaxX = annRender.x + annRender.calcWidth;
					
					for (int k = i + 1; k < model.getRowCount(); k++) {
						depTier = model.getRowData(k);
						int depTierHeight = depTier.getRenderInfo().height;
						if (depTier.hasAncestor(tier)) {
							tempY += vrInfo.vertLineMargin;
							tempY += depTierHeight;
							g2d.drawString(model.getShortTierNameForIndex(k), xOff, tempY - 2);
							
							//render annotations on the dep. tier
							for (int a = 0; a < depTier.getAnnotations().size(); a++) {
								igtAnn = depTier.getAnnotations().get(a);
								annRender =igtAnn.getRenderInfo();
								if (annRender.x < chMinX) {
									continue;
								}
								if (annRender.x > chMaxX) {
									break;
								}
								
								g2d.drawString(igtAnn.getTextValue(), xOff + vrInfo.headerWidth + annRender.x, 
										tempY - 2);// TODO position the text correctly vertically
								if (debug) {
									g2d.setColor(DEBUG_BORDER_COLOR);
									g2d.drawRect(xOff + vrInfo.headerWidth + annRender.x, tempY - depTierHeight, 
											annRender.calcWidth, depTierHeight);
									g2d.setColor(vrInfo.foregroundColor);
								}
							}
							
							renderedTier.add(depTier);
						} else {
							// if we assume that the depending tiers are following the "word" level tier immediately
							// we could break here
							break;
						}
					}
					
					tempY += vrInfo.vertLineMargin;
				}
				
				// update i such that the dependent children are skipped?? TODO support a tier order list and a list of visible tiers
			} else { // render root and association tier
				tempY += tierHeight; 
				g2d.drawString(model.getShortTierNameForIndex(i), xOff, tempY - 2); // position the baseline of the text correctly
				if (tier.getRenderInfo().numLines > 1) {
					if (tier.getAnnotations().size() >= 1) {
						igtAnn = tier.getAnnotations().get(0);
						List<String> lines = igtAnn.getRenderInfo().getWrappedLines();
						if (lines != null) {
							for (int a = 0; a < lines.size(); a++) {
								g2d.drawString(lines.get(a), xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - 2);
								
								if (debug) {
									g2d.setColor(DEBUG_BORDER_COLOR);
									g2d.drawRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - tierHeight, 
											igtAnn.getRenderInfo().calcWidth, tierHeight);
									g2d.setColor(vrInfo.foregroundColor);
								}
								
								if (a != lines.size() - 1) {
									tempY += vrInfo.vertLineMargin;
									tempY += tierHeight;
								}
							}
						} //else {//test code
//							g2d.drawString(igtAnn.getTextValue(), xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - 2);
//							if (debug) {
//								g2d.setColor(DEBUG_BORDER_COLOR);
//								g2d.drawRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - vrInfo.tierHeight, 
//										igtAnn.getRenderInfo().calcWidth, vrInfo.tierHeight);
//								g2d.setColor(vrInfo.foregroundColor);
//							}
//						}					
					} 
				} else {
					if (tier.getAnnotations().size() >= 1) {
						igtAnn = tier.getAnnotations().get(0);
						g2d.drawString(igtAnn.getTextValue(), xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - 2);
						if (debug) {
							g2d.setColor(DEBUG_BORDER_COLOR);
							g2d.drawRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - tierHeight, 
									igtAnn.getRenderInfo().calcWidth, tierHeight);
							g2d.setColor(vrInfo.foregroundColor);
						}
					}
				}
			}
			renderedTier.add(tier);
			// render annotations?
			tempY += vrInfo.vertLineMargin;
		}
	}
		
	/**
	 * Renders a row / group into the specified graphics context.
	 */
	public void renderRow2(Graphics2D g2d, IGTDataModel model, IGTViewerRenderInfo vrInfo, int row,
			int xOff, int yOff, int width, int height) {
		// clear the rectangle for this row
		if (row % 2 == 0) {
			g2d.setColor(vrInfo.backgroundColor);
		} else {
			g2d.setColor(vrInfo.backgroundColor2);
		}
		g2d.fillRect(xOff, yOff, width, height);
		
		g2d.setColor(vrInfo.foregroundColor);
		g2d.drawLine(vrInfo.headerWidth, yOff, vrInfo.headerWidth, yOff + height);
		
		// start rendering tiers, tier labels, annotations
		int tempY = yOff;
		IGTTier tier;
		IGTAnnotation igtAnn;
		IGTNodeRenderInfo annRender;
				
		List<IGTTier> renderedTier = new ArrayList<IGTTier>();
		for (int i = 0; i < model.getRowCount(); i++) {
			tier = model.getRowData(i);
			if (renderedTier.contains(tier)) {
				continue;
			}	
			
			if (tier.isInWordLevelBlock()) {
				// tier labels
				g2d.setFont(vrInfo.defaultFont);
				g2d.setColor(vrInfo.getColorForTier(tier));
				if (tier.getRenderInfo().getYPositions() != null) {
					for (int j = 0; j < tier.getRenderInfo().getYPositions().size(); j++) {
						// TODO position the baseline of the text correctly
						tempY = yOff + tier.getRenderInfo().getYPositions().get(j) + tier.getRenderInfo().height;
						// TODO use the font from the tier render info 
						//g2d.setFont(tier.getRenderInfo().tierFont);
						g2d.drawString(model.getShortTierNameForIndex(i), xOff, tempY - 2);
					}
				} else {
					tempY = yOff + tier.getRenderInfo().y + tier.getRenderInfo().height;
					// TODO use the font from the tier render info 
					//g2d.setFont(tier.getRenderInfo().tierFont);
					g2d.drawString(model.getShortTierNameForIndex(i), xOff, tempY - 2);
				}
				
				// annotations
				g2d.setColor(vrInfo.foregroundColor);
				g2d.setFont(vrInfo.getFontForTier(tier.getTierName()));
				for (int j = 0; j < tier.getAnnotations().size(); j++) {
					igtAnn = tier.getAnnotations().get(j);
					annRender = igtAnn.getRenderInfo();
					tempY = yOff + annRender.y + annRender.height;
					
					g2d.drawString(igtAnn.getTextValue(), xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, 
							tempY - 2);// TODO position the text correctly vertically
					if (debug) {
						g2d.setColor(DEBUG_BORDER_COLOR);
						g2d.drawRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, yOff + annRender.y, 
								igtAnn.getRenderInfo().calcWidth, annRender.height);
						g2d.setColor(vrInfo.foregroundColor);
					}
				}
			} else {
				// other types of tiers
				tempY = yOff + tier.getRenderInfo().y + tier.getRenderInfo().height;
				
				// tier labels
				g2d.setFont(vrInfo.defaultFont);
				g2d.setColor(vrInfo.getColorForTier(tier));
				g2d.drawString(model.getShortTierNameForIndex(i), xOff, tempY - 2); // position the baseline of the text correctly
				
				//annotation
				
				if (tier.getRenderInfo().numLines > 1) {
					if (tier.getAnnotations().size() >= 1) {
						igtAnn = tier.getAnnotations().get(0);
						List<String> lines = igtAnn.getRenderInfo().getWrappedLines();
						
						if (lines != null) {
							g2d.setFont(vrInfo.getFontForTier(tier.getTierName()));
							g2d.setColor(vrInfo.foregroundColor);
							for (int a = 0; a < lines.size(); a++) {
								g2d.drawString(lines.get(a), xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - 2);
								
								if (debug) {
									g2d.setColor(DEBUG_BORDER_COLOR);
									g2d.drawRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - tier.getRenderInfo().height, 
											igtAnn.getRenderInfo().calcWidth, tier.getRenderInfo().height);
									g2d.setColor(vrInfo.foregroundColor);
								}
								
								if (a != lines.size() - 1) {
									tempY += vrInfo.vertLineMargin;
									tempY += tier.getRenderInfo().height;
								}
							}
						} 				
					} 
				} else {
					if (tier.getAnnotations().size() >= 1) {
						igtAnn = tier.getAnnotations().get(0);
						g2d.setFont(vrInfo.getFontForTier(tier.getTierName()));
						g2d.setColor(vrInfo.foregroundColor);
						g2d.drawString(igtAnn.getTextValue(), xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - 2);
						if (debug) {
							g2d.setColor(DEBUG_BORDER_COLOR);
							g2d.drawRect(xOff + vrInfo.headerWidth + igtAnn.getRenderInfo().x, tempY - tier.getRenderInfo().height, 
									igtAnn.getRenderInfo().calcWidth, tier.getRenderInfo().height);
							g2d.setColor(vrInfo.foregroundColor);
						}
					}
				}
			}
			
			renderedTier.add(tier);
			tempY += vrInfo.vertLineMargin;//??
		}
	}
}
