package mpi.eudico.client.annotator.interlinear.edit;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.SwingUtilities;

import mpi.eudico.client.annotator.interlinear.IGTTierType;
import mpi.eudico.client.annotator.interlinear.edit.actions.IGTDeleteAction;
import mpi.eudico.client.annotator.interlinear.edit.actions.IGTEditAction;
import mpi.eudico.client.annotator.interlinear.edit.actions.IGTInterlinearizeAction;
import mpi.eudico.client.annotator.interlinear.edit.actions.SuggestionSetMockup;
import mpi.eudico.client.annotator.interlinear.edit.event.IGTViewerChangeListener;
import mpi.eudico.client.annotator.interlinear.edit.event.IGTViewerModelEvent;
import mpi.eudico.client.annotator.interlinear.edit.event.IGTViewerModelListener;
import mpi.eudico.client.annotator.interlinear.edit.event.ModelEventType;
import mpi.eudico.client.annotator.interlinear.edit.event.SuggestionSelectionListener;
import mpi.eudico.client.annotator.interlinear.edit.event.SuggestionSetEvent;
import mpi.eudico.client.annotator.interlinear.edit.event.SuggestionSetListener;
import mpi.eudico.client.annotator.interlinear.edit.event.SuggestionSetProvider;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTAnnotation;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTDataModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTSuggestionModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTSuggestionViewerModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTTier;
import mpi.eudico.client.annotator.interlinear.edit.render.IGTViewerRenderInfo;
import nl.mpi.lexan.annotyzers.Mediator;
import nl.mpi.lexan.annotyzers.helpers.Position;
import nl.mpi.lexan.annotyzers.helpers.Suggestion;
import nl.mpi.lexan.annotyzers.helpers.SuggestionSet;

@SuppressWarnings("serial")
public class IGTViewer extends JComponent implements MouseListener, MouseMotionListener,
IGTEditProvider, IGTViewerModelListener, SuggestionSetListener {
	private IGTViewerModel model;
	private IGTViewerRenderInfo viewerRenderInfo;
	private IGTGroupRenderer groupRenderer;
	private IGTGroupEditor groupEditor;
	private Mediator mediator;
	/** A wrapper for the mediator that hides the "input side" of the mediator interface */
	private MediatorOutConnector mediatorOutConnector;
	
	private int verticalScrollValue = 0;
	private List<IGTViewerChangeListener> viewerChangeListeners;
	private ImageIcon resizeIcon;
	private int iconMargin = 2;
	private int dragStartX;
	private boolean dragging = false;

	public IGTViewer(IGTViewerModel model, Mediator mediator) {
		super();
		this.model = model;
		this.mediator = mediator;
		viewerChangeListeners = new ArrayList<IGTViewerChangeListener>(4);
		viewerRenderInfo = new IGTViewerRenderInfo();
		viewerRenderInfo.vertRowMargin = 4;
		viewerRenderInfo.vertLineMargin = 2;
		viewerRenderInfo.whitespaceWidth = 8;
		viewerRenderInfo.headerWidth = 100;
		
		try {
            resizeIcon = new ImageIcon(this.getClass()
                    .getResource("/mpi/eudico/client/annotator/resources/SplitPaneDividerH6.gif"));
		} catch (Exception ex) {// any exception
			System.out.println("No divider icon: " + ex.getMessage());
		}
		// store mappings tier -> FontMetrics
		groupRenderer = new IGTGroupRenderer();
		groupEditor = new IGTGroupEditor(this);
		mediatorOutConnector = new MediatorOutConnector(mediator);
		addMouseListener(this);
		addMouseListener(this);
		model.addIGTViewerModelListener(this);
		// TODO maybe the viewer should not add itself as listener?
		// where and how to remove as listener?
		if (mediator instanceof SuggestionSetProvider) {
			((SuggestionSetProvider) mediator).addSuggestionSetListener(this);
		}
	}
	
	/**
	 * Set the per tier color Map
	 */
	public void setTierColorMap(Map<String, Color> tierColorMap){
		viewerRenderInfo.setTierColorMap(tierColorMap);
	}
	
	/**
	 * Set the per tier font Map
	 */
	public void setTierFontMap(Map<String, java.awt.Font> fontMap){
		viewerRenderInfo.setTierFontMap(fontMap);
	}
	
	/**
	 * Set the font size
	 */
	public void setTierFontSize(int size){
		viewerRenderInfo.setFontSize(size);
	}
	
	/**
	 * In the paint component the group renderer is called to render the visible rows/blocks.
	 * 
	 */
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2d = (Graphics2D) g;
		g2d.setColor(Color.white); // background color
		g2d.fillRect(0, 0, getWidth(), getHeight());
		
		int yAdv = 0;
		for (int i = 0; i < getRowCount(); i++) {
			int h = getRowHeight(i);
			if (yAdv + h <= verticalScrollValue) {
				yAdv += h;
				continue;
			}
			if (yAdv + verticalScrollValue > getHeight()) {
				break;
			}
			
			// paint
			groupRenderer.renderRow2(g2d, model.getRowData(i), viewerRenderInfo, i,
					0, verticalScrollValue + yAdv, getWidth(), h + viewerRenderInfo.vertRowMargin);
			yAdv += h;
			yAdv += viewerRenderInfo.vertRowMargin;
		}
		// paint resize icon, if not null
		if (resizeIcon != null) {
			g2d.setColor(Color.white); // background color
			g2d.fillRect(viewerRenderInfo.headerWidth - resizeIcon.getIconWidth() - 2 * iconMargin, 0, 
					resizeIcon.getIconWidth() + 2 * iconMargin, resizeIcon.getIconHeight() + 2 * iconMargin);
			resizeIcon.paintIcon(this, g2d, viewerRenderInfo.headerWidth - resizeIcon.getIconWidth() - iconMargin, iconMargin);
		}
	}

	/**
	 * Returns the height of the requested row.
	 *  
	 * @param row row index
	 * @return the height of the row.
	 */
	public int getRowHeight(int row) {
		try {
			return model.getRowData(row).getRenderInfo().height;
		} catch (ArrayIndexOutOfBoundsException ai) {
			// log error
			return 0;
		}			
	}
	
	/**
	 * Returns the number of rows.
	 * 
	 * @return the number of rows in the viewer
	 */
	public int getRowCount() {
		return model.getRowCount();
	}
	
	/**
	 * Returns the total height of all the data in the model.
	 * 
	 * @return the total height
	 */
	public int getTotalHeight() {
		if (model.getRowCount() == 0) {
			return 0;
		}
		int totHeight = 0;
		for (int i = 0; i < model.getRowCount(); i++) {
			totHeight += getRowHeight(i);
			totHeight += viewerRenderInfo.vertRowMargin;
		}
		
		return totHeight;
	}
	
	/**
	 * Get the current scroll value.
	 * 
	 * @return the verticalScrollValue
	 */
	public int getVerticalScrollValue() {
		return verticalScrollValue;
	}

	/**
	 * Sets the vertical scroll value.
	 * 
	 * @param verticalScrollValue the verticalScrollValue to set
	 */
	public void setVerticalScrollValue(int verticalScrollValue) {
		int oldScrollValue = this.verticalScrollValue;
		this.verticalScrollValue = verticalScrollValue;
		// check if there is an editor active
		if (groupEditor.getParent() == this) {
			int diff = oldScrollValue - this.verticalScrollValue;
			Rectangle bounds = groupEditor.getBounds();
			bounds.y -= diff;
			groupEditor.setBounds(bounds);
		}
			
		repaint();
	}

	/**
	 * Initiates a calculation of the height of one row (i.e. one block of annotations,
	 * a top level annotation with all its depending annotations).
	 * 
	 * @param row the row (== root annotation plus depending annotations) to calculate the height for
	 */
	public void calculateHeightForRow(int row) {
		try {
			int width = getWidth() - viewerRenderInfo.headerWidth;
			if (width > 0) {// introduce a minimum width for the data
				IGTDataModel dModel = model.getRowData(row);
				Graphics2D g2d = (Graphics2D) getGraphics();
				// TODO start calculating width at top level of the group
				g2d.setFont(viewerRenderInfo.defaultFont);
				IGTCalculator.calculateShortTierLabels(g2d, dModel, viewerRenderInfo.headerWidth);
				// calculates everything
				IGTTier rootTier = dModel.getRootRow();
				/*int rootWidth =*/ IGTCalculator.calculateTierAndAnnotationsLAS(g2d, viewerRenderInfo, 
						rootTier, true);
				
				// calculate wrapping information
				int tierCount = dModel.getRowCount();
				IGTTier curTier;
				List<String> procTiers = new ArrayList<String>(); // add already processed tiers to the list
				for (int i = 0; i < tierCount; i++) {
					curTier = dModel.getRowData(i);
					if (procTiers.contains(curTier.getTierName())) {
						continue;
					}
					
					IGTCalculator.calculateWrappingInfo(g2d, viewerRenderInfo, curTier, width);
					
					procTiers.add(curTier.getTierName());
					if (curTier.getType() == IGTTierType.WORD_LEVEL_ROOT || curTier.getType() == IGTTierType.SUBDIVISION) {
						addChildTiers(procTiers, curTier);
					}
				}

				// hier TODO collect information for total height and store it in the render info object
				// of the row
				int rowHeight = 0;
				tierCount = dModel.getRowCount();
				procTiers.clear();
				for (int i = 0; i < tierCount; i++) {
					curTier = dModel.getRowData(i);
					if (procTiers.contains(curTier.getTierName())) {
						continue;
					}
					// TODO get the real tier font height
					if (curTier.getType() == IGTTierType.WORD_LEVEL_ROOT || 
							curTier.getType() == IGTTierType.SUBDIVISION) {
						int subHeight = getTierHeightRecursive(g2d, curTier);
						int numLineBlocks = 1;
						if (curTier.getRenderInfo().getWrapIndices() != null) {
							numLineBlocks = curTier.getRenderInfo().getWrapIndices().size() + 1;
						}
//						rowHeight += (curTier.getRenderInfo().numLines * subHeight);
						rowHeight += (numLineBlocks * subHeight);
						rowHeight += ((curTier.getRenderInfo().numLines - 1) * viewerRenderInfo.vertLineMargin);
						addChildTiers(procTiers, curTier);
					} else {
						rowHeight += (curTier.getRenderInfo().numLines * viewerRenderInfo.getHeightForTier(g2d, curTier.getTierName()));
						rowHeight += ((curTier.getRenderInfo().numLines - 1) * viewerRenderInfo.vertLineMargin);
					}
					procTiers.add(curTier.getTierName());
					rowHeight += viewerRenderInfo.vertLineMargin;
				}
				
				dModel.getRenderInfo().height = rowHeight;
				
				int rowHeight2 = 0;
				//int yPosStart = 0;
				procTiers.clear();
				for (int i = 0; i < tierCount; i++) {
					curTier = dModel.getRowData(i);
					if (procTiers.contains(curTier.getTierName())) {
						continue;
					}
					// TODO get the real tier font height
					if (curTier.getType() == IGTTierType.WORD_LEVEL_ROOT || 
							curTier.getType() == IGTTierType.SUBDIVISION) {
						int subHeight = IGTCalculator.calculateTierYPositionRecursive(g2d, viewerRenderInfo, 
								curTier, rowHeight2);
						rowHeight2 += subHeight;
						addChildTiers(procTiers, curTier);
					} else {
						int subHeight = IGTCalculator.calculateTierYPosition(g2d, viewerRenderInfo, 
								curTier, rowHeight2);
						rowHeight2 += subHeight;
						
						//rowHeight2 += ((curTier.getRenderInfo().numLines - 1) * viewerRenderInfo.vertLineMargin);
					}
					procTiers.add(curTier.getTierName());
					//rowHeight2 += viewerRenderInfo.vertLineMargin;
				}
				
				dModel.getRenderInfo().height = rowHeight2;
				//System.out.println("H1: " +rowHeight + " - H2: " + rowHeight2);
				notifyViewerChangeListener();
			}
		} catch (ArrayIndexOutOfBoundsException ai) {
			// log error
		}
	}
	
	public void recalculateAllRows() {
		for (int i = 0; i < getRowCount(); i++) {
			calculateHeightForRow(i);
			IGTDataModel rowModel = model.getRowData(i);
			if (i == 0) {
				rowModel.getRenderInfo().y = viewerRenderInfo.vertRowMargin;
			} else {
				IGTDataModel rowModel2 = model.getRowData(i - 1);
				rowModel.getRenderInfo().y = rowModel2.getRenderInfo().y + 
						rowModel2.getRenderInfo().height + viewerRenderInfo.vertRowMargin;
			}
		}
		// check if there is an editor active
		if (model.getEditingRow() > -1) {
			IGTDataModel rowModel = model.getRowData(model.getEditingRow());
			groupEditor.setBounds(0, rowModel.getRenderInfo().y - 2 * viewerRenderInfo.vertRowMargin, 
					getWidth(), rowModel.getRenderInfo().height + viewerRenderInfo.vertRowMargin);
			groupEditor.editIGTGroup(rowModel, viewerRenderInfo, model.getEditingRow(), 0, viewerRenderInfo.vertRowMargin);
		}
		notifyViewerChangeListener();
	}
	
	/**
	 * Adds the child tiers to the specified list, recursively.
	 * 
	 * @param children the list to add to
	 * @param igtTier the tier to retrieve the children from
	 */
	private void addChildTiers(List<String> children, IGTTier igtTier) {
		if (igtTier.getChildTiers() != null && igtTier.getChildTiers().size() > 0) {
			IGTTier child;
			for (int i = 0; i < igtTier.getChildTiers().size(); i++) {
				child = igtTier.getChildTiers().get(i);
				children.add(child.getTierName());
				addChildTiers(children, child);
			}
		}
	}
	
	private int getTierHeightRecursive(Graphics2D g2d, IGTTier tier) {
		int curHeight = viewerRenderInfo.getHeightForTier(g2d, tier.getTierName()); 
		curHeight += viewerRenderInfo.vertLineMargin;
		if (tier.getChildTiers() != null) {
			IGTTier chTier;
			for (int i = 0; i < tier.getChildTiers().size(); i++) {
				chTier = tier.getChildTiers().get(i);
				curHeight += getTierHeightRecursive(g2d, chTier);
			}
		}
		
		return curHeight;
	}

	/**
	 * 
	 * @param yLocation
	 * @return the row index or -1 of there is no row at the given location
	 */
	private int getRowIndexForLocation(int yLocation) {
		IGTDataModel rowModel;
		
		for (int i = 0; i < model.getRowCount(); i++) {
			rowModel = model.getRowData(i);
			if (yLocation >= rowModel.getRenderInfo().y && 
					yLocation <= rowModel.getRenderInfo().y + rowModel.getRenderInfo().height) {
				return i;
			}			
		}
		
		return -1;
	}

	@Override
	public void mouseClicked(MouseEvent me) {
		// TODO first stop an editing row if there is one
		int row = getRowIndexForLocation(me.getPoint().y - verticalScrollValue);
		
		if (row >= 0) {
			IGTDataModel rowModel = model.getRowData(row);
			model.startEditingRow(row);// ?? is this necessary?
			add(groupEditor);
			groupEditor.setBounds(0, rowModel.getRenderInfo().y - 2 * viewerRenderInfo.vertRowMargin + verticalScrollValue, 
					getWidth(), rowModel.getRenderInfo().height + (int)(1.5 * viewerRenderInfo.vertRowMargin));
			groupEditor.editIGTGroup(rowModel, viewerRenderInfo, row, 0, viewerRenderInfo.vertRowMargin);
			// calculate the mouse position in the editor
			int nx = me.getX();
			int ny = me.getY() - rowModel.getRenderInfo().y - verticalScrollValue;
			if ((SwingUtilities.isRightMouseButton(me) && (me.getButton() == MouseEvent.BUTTON1 ^ me.isMetaDown())) 
	                || me.isPopupTrigger()) {
	        	
				groupEditor.pressedAt(nx, ny);
	        } else {
	        	groupEditor.clickedAt(nx, ny);
	        }
		} else {
			remove(groupEditor);
			repaint();
		}
		
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {
		// stub
		
	}


	@Override
	public void mouseExited(MouseEvent arg0) {
		// stub
		
	}


	@Override
	public void mousePressed(MouseEvent me) {
		//System.out.println("Start: " + me.getX());
		if (me.getX() >= viewerRenderInfo.headerWidth - 3 && me.getX() <= viewerRenderInfo.headerWidth + 3) {
			dragStartX = me.getX();
			dragging = true;
		}
		
	}


	@Override
	public void mouseReleased(MouseEvent me) {
		//System.out.println("Release: " + me.getX());
		if (dragging) {
			dragging = false;
			// update the header width
			int distance = me.getX() - dragStartX;
			if (distance != 0) {
				viewerRenderInfo.headerWidth = viewerRenderInfo.headerWidth + distance;
				recalculateAllRows();
				repaint();
			}			
		}
		
	}
	

	@Override
	public void mouseDragged(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseMoved(MouseEvent arg0) {
		// stub		
	}

// ############## IGTEditProvider implementation  ###############################
	@Override
	public List<IGTEditAction> actionsForAnotation(IGTAnnotation igtAnnotation) {
		if (igtAnnotation != null) {
			// TODO check type of annotation, based on that create a list of actions
			
			// for now
			List<IGTEditAction> actions = new ArrayList<IGTEditAction>(2);
			
			IGTInterlinearizeAction act1 = new IGTInterlinearizeAction(igtAnnotation, mediatorOutConnector,
					"Interlinearize");
			actions.add(act1);
			
			SuggestionSetMockup act3 = new SuggestionSetMockup(igtAnnotation, this,
					"Make suggestions");
			actions.add(act3);
			
			IGTDeleteAction act2 = new IGTDeleteAction(igtAnnotation, mediatorOutConnector, 
					"Delete");
			actions.add(act2);
			
			return actions;
		}
		return null;
	}


// ################# IGTViewerModelListener implementation ################################
	@Override
	public void viewerModelChanged(IGTViewerModelEvent event) {
		if (event != null) {
			if (event.getRow() > -1) {
				if (event.getType() == ModelEventType.CHANGE) {
					// when the interlinearize action is called on the entire tier, 
					//there is no editor available
					calculateHeightForRow(event.getRow());// triggers an event
					if (event.getRow() == model.getEditingRow()) {
						// notify the groupEditor
						//calculateHeightForRow(event.getRow());// triggers an event
						// if the editor is there, resize  if necessary
						if (groupEditor.getParent() == this) {
							IGTDataModel rowModel = model.getRowData(event.getRow());
							groupEditor.setBounds(0, rowModel.getRenderInfo().y - 2 * viewerRenderInfo.vertRowMargin + verticalScrollValue, 
									getWidth(), rowModel.getRenderInfo().height + (int)(1.5 * viewerRenderInfo.vertRowMargin));
						}
					}
				} 
			}
		}		
	}

	/**
	 * If an annotator/recognizer produces a list of suggestions, some user interaction is 
	 * needed to select one of the provided suggestions.
	 * 
	 * For now the event of new suggestion becoming available will start editing the group of
	 * annotations (the row) the suggestions concern.
	 * 
	 * @param event the event holding the list of suggestions
	 */
	@Override
	public void suggestionSetDelivered(SuggestionSetEvent event) {
		if (event != null && event.getSuggestionSets() != null) {
			List<SuggestionSet> sugSets = event.getSuggestionSets();
			Position pos = null;
			for (SuggestionSet sugSet : sugSets) {
				if (sugSet.suggestions.size() > 0) {
					Suggestion sug = sugSet.suggestions.get(0);
					pos = sug.position;
					// find the row to edit based on tier name and time information
					int rowIndex = getRowIndexForTierAndTime(pos.tierId, (pos.beginTime + pos.endTime) / 2);
					
					if (rowIndex > -1) {
						// start editing the row, if it isn't already the editing row, and inform the 
						// editor. In most cases all suggestion sets will pertain to one row (i.e. group of annotations)
						
						if (model.getEditingRow() != rowIndex) {
							// set editing row, scroll if needed
							groupEditor.editIGTGroup(model.getRowData(rowIndex), viewerRenderInfo, rowIndex, 
									0, viewerRenderInfo.vertRowMargin);
							// etc
							break;
						}
					}
				}
			}
			
			// create a suggestion model
			IGTSuggestionViewerModel svModel = new IGTSuggestionViewerModel();
			for (SuggestionSet ss : sugSets) {
				IGTSuggestionModel sugModel = new IGTSuggestionModel(ss);
				svModel.addRow(sugModel);
			}
			
			groupEditor.suggestionSetDelivered(svModel, pos, (SuggestionSelectionListener) mediator);
		}
		
	}
	
	/**
	 * Returns the index of the row of data containing the specified tier and 
	 * with the specified time within its time boundaries.  
	 * 
	 * @param tierName the tier name to look for
	 * @param time the time to find
	 * @return the row index or -1
	 */
	public int getRowIndexForTierAndTime(String tierName, long time) {
		if (tierName != null /*&& time > -1*/) {
			IGTDataModel rowModel;
			
			for (int i = 0; i < model.getRowCount(); i++) {
				rowModel = model.getRowData(i);
				
				if (rowModel.getBeginTime() <= time && 
						rowModel.getEndTime() >= time) {
					if (rowModel.getRowIndexForTier(tierName) > -1) {
						return i;
					}
				}
			}
		}
		
		return -1;
	}

	/**
	 * Adds a viewer change listener.
	 * 
	 * @param listener the listener
	 */
	public void addViewerChangeListener(IGTViewerChangeListener listener) {
		if (!viewerChangeListeners.contains(listener)) {
			viewerChangeListeners.add(listener);
		}
	}

	/**
	 * Removes a viewer change listener.
	 * 
	 * @param listener the listener
	 */
	public void removeViewerChangeListener(IGTViewerChangeListener listener) {
		viewerChangeListeners.remove(listener);
	}
	
	/**
	 * Notifies listeners of a change in the view.
	 */
	private void notifyViewerChangeListener() {
		for (IGTViewerChangeListener listener : viewerChangeListeners) {
			listener.viewChanged();
		}
	}

}
