package mpi.eudico.client.annotator.interlinear.edit.actions;

import java.awt.event.ActionEvent;

import javax.swing.Icon;

import nl.mpi.lexan.annotyzers.helpers.Position;

import mpi.eudico.client.annotator.interlinear.edit.MediatorOutConnector;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTAnnotation;

/**
 * An action class for starting the interlinearization of single annotation.
 *  
 * @author Han Sloetjes
 */
public class IGTInterlinearizeAction extends IGTEditAction {
	/** The mediator to use for starting the interlinearization action */
	protected MediatorOutConnector mediator;
	
	/**
	 * @see IGTEditAction#IGTEditAction(IGTAnnotation, String)
	 * 
	 * @param mediator the mediator connector class providing access to the interlinearization mechanism
	 */
	public IGTInterlinearizeAction(IGTAnnotation igtAnnotation, MediatorOutConnector mediator, String label) {
		super(igtAnnotation, label);
		this.mediator = mediator;
	}
	
	/**
	 * @see IGTEditAction#IGTEditAction(IGTAnnotation, String, Icon)
	 * 
	 * @param mediator the mediator connector class providing access to the interlinearization mechanism
	 */
	public IGTInterlinearizeAction(IGTAnnotation igtAnnotation, MediatorOutConnector mediator, String label,
			Icon icon) {
		super(igtAnnotation, label, icon);
		this.mediator = mediator;
	}

	/**
	 * @see IGTEditAction#IGTEditAction(IGTAnnotation)
	 * @param mediator the mediator connector class providing access to the interlinearization mechanism
	 */
	public IGTInterlinearizeAction(IGTAnnotation igtAnnotation, MediatorOutConnector mediator) {
		super(igtAnnotation);
		this.mediator = mediator;
	}

	/**
	 * Creates a Position object that initializes the interlinearization process.
	 *   
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public void actionPerformed(ActionEvent ae) {
		if (igtAnnotation != null && mediator != null) {
			Position pos = new Position();
			pos.tierId = igtAnnotation.getAnnotation().getTier().getName();
			pos.beginTime = igtAnnotation.getAnnotation().getBeginTimeBoundary();
			pos.endTime = igtAnnotation.getAnnotation().getEndTimeBoundary();
			
			mediator.annotyze(pos);
		}		
	}

}
