package mpi.eudico.client.annotator.interlinear.edit.render;

import java.awt.Point;
import java.util.ArrayList;
import java.util.List;


/**
 * A class for storing rendering information for an annotation node.
 * Coordinates, sizes, number of lines, colors
 *  
 * @author Han Sloetjes
 */
public class IGTNodeRenderInfo extends IGTRenderInfo {
	// TODO add members, might all be public? 
	public int realWidth;
	public int calcWidth;
	
	public int numLines = 1;
	// info about Color and Font on the IGTTier level?
	private List<String> wrappedLines;
	
	/**
	 * Returns the list of wrapped lines. Can be null.
	 * 
	 * @return the list of wrapped lines, can be null
	 */
	public List<String> getWrappedLines() {
		return wrappedLines;
	}
	
	/**
	 * Clears the list of wrapped lines.
	 */
	public void clearWrappedLines() {
		if (wrappedLines != null) {
			wrappedLines.clear();
		}
	}
	
	/**
	 * Adds a line to the list of wrapped lines.
	 * 
	 * @param line the line to add
	 */
	public void addWrappedLine(String line) {
		if (line == null) {
			return;
		}
		
		if (wrappedLines == null) {
			wrappedLines = new ArrayList<String>(4);
		}
		
		wrappedLines.add(line);
	}

	/**
	 * The test is performed on the extended, calculated width of the node/annotation.
	 * 
	 * @see mpi.eudico.client.annotator.interlinear.edit.render.IGTRenderInfo#isPointInRenderArea(java.awt.Point)
	 */
	@Override
	public boolean isPointInRenderArea(Point p) {
		if (p == null) {
			return false;
		}
		return p.x >= x && p.x <= x + calcWidth && 
				p.y >= y && p.y <= y + height;
	}
	
	
}
