package mpi.eudico.client.annotator.interlinear.edit.render;

import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.List;


/**
 * Class for storing rendering information on the tier level.
 * Note: maybe this information should be at the renderer/editor level.
 * 
 * @author Han Sloetjes
 */
public class IGTTierRenderInfo extends IGTRenderInfo {
	public Color textColor;// foreground
	public Color editBgColor;//default background
	public Color nonEditBgColor;// or no annotation background
	
	public Font tierFont;// or store this on the viewer component layer
	
	// these fields are used by "word" level tiers, first subdivision tiers under the root
	/** Stores the indices of the annotations at which to wrap to a next block of lines */
	private List<Integer> blockWrapIndices;
	// in case of block-wise wrapping store the y positions of the top of each wrapped line 
	private List<Integer> yPositions;
	
	public int numLines = 1;
	// stores the total height of non-block wrapped but line wrapped tiers
	public int renderHeight;
	
	/**
	 * Returns the list of indices at which to wrap the blocks of annotations. Can be null.
	 * 
	 * @return the list of annotation indices at which to wrap, can be null
	 */
	public List<Integer> getWrapIndices() {
		return blockWrapIndices;
	}
	
	/**
	 * Returns the list of y positions for this tier
	 * 
	 * @return the list of y positions
	 */
	public List<Integer> getYPositions() {
		return yPositions;
	}
	
	/**
	 * Removes all wrap indices from the list (if a list exists).
	 */
	public void clearWrapIndices() {
		if (blockWrapIndices != null) {
			blockWrapIndices.clear();
			numLines = 1;
		}
	}
	
	/**
	 * Removes all stored y positions of this tier.
	 */
	public void clearYPositions() {
		if (yPositions != null) {
			yPositions.clear();
			yPositions = null;
		}
	}
	
	/**
	 * Adds the index to the list of wrap indices (at the end). 
	 * 
	 * @param index the new wrap index to add to the list
	 */
	public void addWrapIndex(int index) {
		if (blockWrapIndices == null) {
			blockWrapIndices = new ArrayList<Integer>(4);
		}
		
		blockWrapIndices.add(index);
		numLines = blockWrapIndices.size() + 1;
	}
	
	/**
	 * Adds an y position to the list.
	 * 
	 * @param yPos an y position
	 */
	public void addYPosition(int yPos) {
		if (yPositions == null) {
			yPositions = new ArrayList<Integer>(4);
		}
		yPositions.add(yPos);
	}
}
