package mpi.eudico.client.annotator.turnsandscenemode;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.Action;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;

import mpi.eudico.client.annotator.Constants;
import mpi.eudico.util.TimeFormatter;

/**
 * A panel that serves as the basis for both a cell renderer and a cell editor.
 * Attempts are made to keep this a view object as much as possible.
 * 
 * @author Han Sloetjes
 */
@SuppressWarnings("serial")
public class TaSCellPanel extends JPanel {
	private JLabel timeLabel;
	private JLabel speakerLabel;
	private JTextArea textArea;
	private DecorationPanel indicatorPanel;
	/* added assuming the panel would be in a JList. In a JTable this can be removed (there is a "showHorizontalLines" method) */
	private JPanel linePanel;
	private Color DEF_PANEL_BG;
	private Color DEF_TEXT_BG;
	public final Color NO_ANN_BG = new Color(230,230,250);
	
	private boolean timeLabelVisible;
	private boolean speakerLabelVisible;
	
	private TaSAnno curAnnoSegment;
	
	/**
	 * Constructor, creates and initializes the ui components.
	 */
	public TaSCellPanel() {
		super();
		initComponents();
	}
	
	/**
	 * Sets the annotation wrapper object to render.
	 * 
	 * @param slAnno the annotation wrapper object
	 */
	public void setTaSAnnotation(TaSAnno slAnno) {
		curAnnoSegment = slAnno;
		
		if (curAnnoSegment != null) {
			textArea.setText(curAnnoSegment.getText());
			if (slAnno.getAnnotation() == null) {
				textArea.setBackground(Constants.EVEN_ROW_BG);// NO_ANN_BG
			} else {
				textArea.setBackground(DEF_TEXT_BG);
			}
			long et = curAnnoSegment.getEndTime();
			String etLabel = null;
			if (et == Long.MAX_VALUE) {
				etLabel = "?";
			} else {
				etLabel = TimeFormatter.toString(et);
			}
			timeLabel.setText(TimeFormatter.toString(curAnnoSegment.getBeginTime()) + " - " + 
					etLabel);
			if (curAnnoSegment.getParticipant() != null) {
				if (curAnnoSegment.getParticipant().length() > 3) {
					speakerLabel.setText(curAnnoSegment.getParticipant().substring(0, 3));
				} else {
					speakerLabel.setText(curAnnoSegment.getParticipant());
				}
			} else {
				speakerLabel.setText("");
			}
		} else {
			textArea.setText("");
			timeLabel.setText("");
			speakerLabel.setText("");
			textArea.setBackground(DEF_TEXT_BG);
		}
	}
	
	private void initComponents() {
		setLayout(new GridBagLayout());
		timeLabelVisible = true;
		speakerLabelVisible = true;
		
		timeLabel = new JLabel("T");
		speakerLabel = new JLabel("S");
		
		//indicatorPanel = new JPanel();// => custom panel to paint selection, active annotation and media time colors?
		indicatorPanel = new DecorationPanel();
		indicatorPanel.setPreferredSize(new Dimension(14, 20));
		DEF_PANEL_BG = indicatorPanel.getBackground();
		linePanel = new JPanel(null);
		linePanel.setPreferredSize(new Dimension(1, 1));
		
		//textArea = new JTextArea();
		textArea = new TaSTextArea();
		textArea.setLineWrap(true);
		textArea.setWrapStyleWord(true);
		textArea.setMargin(new Insets(0, 3, 0, 3));
		textArea.setEditable(true);
		textArea.setMargin(new Insets(4, 6, 4, 4));
		DEF_TEXT_BG = textArea.getBackground();
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.VERTICAL;
		gbc.gridheight = 2;
		gbc.weighty = 1.0;
		this.add(indicatorPanel, gbc);
		
		gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.NONE;
		gbc.gridx = 1;
		gbc.gridy = 0;
		gbc.gridwidth = 1;
		gbc.gridheight = 2;
		gbc.weightx = 0.0;
		gbc.weighty = 0.0;
		gbc.insets = new Insets(6, 3, 2, 4);
		this.add(speakerLabel, gbc);
		
		gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 2;
		gbc.gridy = 0;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.insets = new Insets(2, 3, 2, 2);
		this.add(textArea, gbc);
		
		gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridx = 2;
		gbc.gridy = 1;
		gbc.gridwidth = 2;
		gbc.gridheight = 1;
		gbc.weightx = 1.0;
		gbc.insets = new Insets(2, 3, 2, 2);
		this.add(timeLabel, gbc);
		
		gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.SOUTHWEST;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridx = 0;
		gbc.gridy = 2;
		gbc.gridheight = 1;
		gbc.gridwidth = 3;
		gbc.weightx = 1.0;
		this.add(linePanel, gbc);	
	}
	
	private void updateLayout() {
		speakerLabel.setVisible(speakerLabelVisible);
		timeLabel.setVisible(timeLabelVisible);
	}
	
	public void startEditing() {
		textArea.requestFocus();
		textArea.setCaretPosition(0);
	}
	
	public void stopEditing() {
		textArea.transferFocusUpCycle();
	}
	
	public void setSelected(boolean selected) {
		if (selected) {
			indicatorPanel.setBackground(Constants.SELECTIONCOLOR);
		} else {
			indicatorPanel.setBackground(DEF_PANEL_BG);
		}
	}
	
	@Override
	public void setFont(Font font) {
		if (font != null) {
			if (textArea != null) {
				textArea.setFont(font);
			}
			if (speakerLabel != null) {
				speakerLabel.setFont(font);
			}
			if (timeLabel != null) {
				int baseSize = font.getSize();
				float timeSize = 10 + 2 * (baseSize / 10f);
				timeLabel.setFont(font.deriveFont(timeSize));
			}	
		}
	}

	public boolean isTimeLabelVisible() {
		return timeLabelVisible;
	}

	public void setTimeLabelVisible(boolean timeLabelVisible) {
		if (this.timeLabelVisible != timeLabelVisible) {
			this.timeLabelVisible = timeLabelVisible;
			updateLayout();
		}
	}

	public boolean isSpeakerLabelVisible() {
		return speakerLabelVisible;
	}

	public void setSpeakerLabelVisible(boolean speakerLabelVisible) {
		if (this.speakerLabelVisible != speakerLabelVisible) {
			this.speakerLabelVisible = speakerLabelVisible;
			updateLayout();
		}
	}

	/**
	 * Gives access to the text area so that actions and listeners can be added etc.
	 * @return the text area
	 */
	public JTextArea getTextArea() {
		return textArea;
	}

	// add method to set decoration flags for time selection (light blue), cross hair (red) and 
	// active annotation (dark blue)
	public void setDecorations(boolean inTimeSelection, boolean containsCrosshair, 
			boolean isActiveAnnotation) {
		indicatorPanel.timeSelected = inTimeSelection;
		indicatorPanel.crosshairInSegment = containsCrosshair;
		indicatorPanel.activeAnnotation = isActiveAnnotation;
	}
	
	@SuppressWarnings("serial")
	private class DecorationPanel extends JPanel {
		boolean timeSelected = true;
		boolean crosshairInSegment = true;
		boolean activeAnnotation = true;
		
		int chHeight = 10;
		/**
		 * 
		 */
		public DecorationPanel() {
			super();
		}

		@Override
		protected void paintComponent(Graphics g) {
			super.paintComponent(g);
			
			Graphics2D g2d = (Graphics2D) g;
			int w = getWidth();
			int h = getHeight();
			if (!timeSelected) {
				g2d.setColor(DEF_PANEL_BG);
			} else {
				g2d.setColor(Constants.SELECTIONCOLOR);
			}
			g2d.fillRect(0, 0, w, h);
			
			if (crosshairInSegment) {
				g2d.setColor(Constants.CROSSHAIRCOLOR);
				g2d.fillRect(0, (h - chHeight) / 2, w, chHeight);
				g2d.drawLine(0, 0, w - 1, 0);
				g2d.drawLine(0, h - 1, w - 1, h - 1);
			}
			
//			if (activeAnnotation) {
//				g2d.setColor(Constants.ACTIVEANNOTATIONCOLOR);
//				g2d.drawRect(0, 0, w - 1, h - 1);
//			}
		}
		
	}
	
	/**
	 * Temporary fix for e.g. Shift+Space as the key stroke for play selection.
	 * This could be generalized maybe...
	 * Nov 2015 added some additional code to change the behavior of "cut": commit the change after cutting text 
	 * but maybe without automatically activating another row in the table? 
	 */
	private class TaSTextArea extends JTextArea {
		int lastKeyCode;
		@Override
		protected void processKeyEvent(KeyEvent e) {
			// int mask = e.getModifiersEx() & InputEvent.SHIFT_DOWN_MASK;
			// combination with Ctrl either work anyway or have a more important role in text editing than
			// in media navigation? Combinations with arrow left/right also have a meaning in text editing
			int mask = e.getModifiersEx() & (InputEvent.SHIFT_DOWN_MASK | /*InputEvent.CTRL_DOWN_MASK |*/ InputEvent.ALT_DOWN_MASK);

			if (e.getKeyCode() == KeyEvent.VK_SPACE) {
				if (mask != 0) {
					if (e.getID() == KeyEvent.KEY_PRESSED) {
						// check if there is a registered KeyStroke - Action combination
						Object key = null;
						if ( (key = getInputMap().get(KeyStroke.getKeyStrokeForEvent(e))) != null) {
							Action act = getActionMap().get(key);
							if (act != null) {
								act.actionPerformed(null);
							}
						}
					}
					
					e.consume();
				}
			} else if (e.getKeyCode() == KeyEvent.VK_UNDEFINED) {
				// workaround to prevent a white space to be added to the document if space
				if (mask != 0 && lastKeyCode == KeyEvent.VK_SPACE && e.getID() == KeyEvent.KEY_TYPED) {
					e.consume();
				}
			}
			
			lastKeyCode = e.getKeyCode();
			super.processKeyEvent(e);
		}
		@Override
		public void cut() {
			super.cut();
			// commit change
			Action cutAction = getActionMap().get("cutPost");
			if (cutAction != null) {
				cutAction.actionPerformed(null);
			}
			
		}
		@Override
		public void paste() {
			super.paste();
			// commit change
			Action pasteAction = getActionMap().get("pastePost");
			if (pasteAction != null) {
				pasteAction.actionPerformed(null);
			}
		}
		
		
	}
}
