package mpi.eudico.client.annotator.turnsandscenemode;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

import mpi.eudico.client.annotator.Constants;
import mpi.eudico.client.annotator.ElanLocale;

/**
 * A small panel for changing the font size of the viewer in this mode.
 * Maybe to be replaced by a more extensive configuration panel/dialog.
 * It currently has Plus and Minus buttons to increase or decrease the font size.
 */
public class TaSFontSizePanel extends JPanel implements ActionListener {
	private TurnsAndSceneViewer viewer;
	private JLabel fontLabel;
	private JButton plusButton;
	private JButton minusButton;
	
	public TaSFontSizePanel(TurnsAndSceneViewer viewer) {
		this.viewer = viewer;
		initComponents();
	}

	private void initComponents() {
		setLayout(new GridBagLayout());
		fontLabel = new JLabel();
		plusButton = new JButton();
		minusButton = new JButton();
		
		try {
			ImageIcon icon = new ImageIcon(this.getClass().getResource("/mpi/eudico/client/annotator/resources/Plus16.gif"));
			plusButton.setIcon(icon);
		} catch (Throwable t) {
			plusButton.setText("+");
		}
		try {
			ImageIcon icon = new ImageIcon(this.getClass().getResource("/mpi/eudico/client/annotator/resources/Minus16.gif"));
			minusButton.setIcon(icon);
		} catch (Throwable t) {
			minusButton.setText("-");
		}
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.WEST;
		gbc.insets = new Insets(2, 2, 2, 2);
		add(fontLabel, gbc);
		
		gbc.gridx = 1;
		gbc.fill = GridBagConstraints.NONE;
		gbc.weightx = 0.0;
		add(minusButton, gbc);
		gbc.gridx = 2;
		add(plusButton, gbc);
		
		updateLocale();
		plusButton.addActionListener(this);
		minusButton.addActionListener(this);
	}
	
	public void updateLocale() {
		fontLabel.setText(ElanLocale.getString("Menu.View.FontSize"));
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		int currentSize = viewer.getFontSize();
		int index = getFontIndex(currentSize);
		
		if (index > -1) {
			if (e.getSource() == plusButton) {
				index++;
				if (index < Constants.FONT_SIZES.length) {
					viewer.setFontSize(Constants.FONT_SIZES[index]);
				}
				if (index == Constants.FONT_SIZES.length - 1) {
					plusButton.setEnabled(false);
				}
				minusButton.setEnabled(true);
			} else if (e.getSource() == minusButton) {
				index--;
				if (index >= 0) {
					viewer.setFontSize(Constants.FONT_SIZES[index]);
				}
				if (index == 0) {
					minusButton.setEnabled(false);
				}
				plusButton.setEnabled(true);
			}
		}
	}
	
	/**
	 * The font sizes are taken from an array in {@link Constants}
	 * @param fontSize the size to retrieve the index for
	 * @return the index
	 */
	private int getFontIndex(int fontSize) {
		int index = -1;
		for (int i = 0; i < Constants.FONT_SIZES.length; i++) {
			if (fontSize == Constants.FONT_SIZES[i]) {
				index = i;
				break;
			}
		}
		
		if (index == -1) {// if the font size was not one of the predefined sizes
			for (int i = 0; i < Constants.FONT_SIZES.length - 1; i++) {
				if (fontSize > Constants.FONT_SIZES[i] && fontSize < Constants.FONT_SIZES[i + 1]) {
					int dif1 = fontSize - Constants.FONT_SIZES[i];
					int dif2 = Constants.FONT_SIZES[i + 1] - fontSize;
					
					if (i <= Constants.FONT_SIZES.length / 2) {
						index = i + 1;// "round up"
					} else {
						if (dif1 <= dif2) {
							index = i;
						} else {
							index = i + 1;
						}
					}
					break;
				}
			}
		}
		
		return index;
	}

}
