package mpi.eudico.client.annotator.turnsandscenemode;

import java.awt.Dimension;
import java.awt.GraphicsEnvironment;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.KeyStroke;
import javax.swing.WindowConstants;

import mpi.eudico.client.annotator.ELAN;
import mpi.eudico.client.annotator.ElanFrame2;
import mpi.eudico.client.annotator.ElanLayoutManager;
import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.SIMPLELAN;
import mpi.eudico.client.annotator.ViewerManager2;
import mpi.eudico.client.annotator.commands.CommandAction;
//import mpi.eudico.client.annotator.ElanFrame2.ElanFrameWindowListener;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.commands.ShortcutsUtil;
import mpi.eudico.client.annotator.commands.global.HelpMA;
import mpi.eudico.client.annotator.commands.global.MenuAction;
import mpi.eudico.client.annotator.gui.ElanMenuItem;
import mpi.eudico.client.annotator.layout.PlayerLayoutModel;
import mpi.eudico.client.annotator.linkedmedia.MediaDescriptorUtil;
import mpi.eudico.client.annotator.turnsandscenemode.commands.ExitTaSMA;
import mpi.eudico.client.annotator.turnsandscenemode.commands.NewTaSMA;
import mpi.eudico.client.annotator.turnsandscenemode.commands.OpenTaSMA;
import mpi.eudico.client.annotator.util.ClientLogger;
import mpi.eudico.client.annotator.util.FrameConstants;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;

/**
 * Customized frame for the simplified annotation layout. 
 * Initially this will be a single layer/single tier working mode.
 *
 */
@SuppressWarnings("serial")
public class TaSFrame extends ElanFrame2 {
	JMenuItem helpMI;
	
    //private Transcription transcriptionForThisFrame;
    //private ElanLayoutManager elanLayoutManager;
	//private TurnsAndSceneManager layoutManager;
	//private ViewerManager2 viewerManager;
	private SLWindowListener windowListener;

	public TaSFrame() {
		super();
		applicationName = "Simplified ELAN " + SIMPLELAN.getVersionString();
		setTitle(applicationName);
	}
	
	/**
	 * 
	 * @param path can be a single layer eaf or an audio or video file.
	 */
	public TaSFrame(String path) {
		this();
		
//		System.out.println("File: " + path);
		if (path != null) {
			openEAF(path);
		} else {
			ClientLogger.LOG.info("File path is null");
		}
	}
	
	/**
	 * The Simplified ELAN application is (for the time being) a single window, single document application.
	 * This allows to set the transcription for the application.
	 *  
	 * @param nextTranscription the next transcription to work on
	 */
	@Override
	public void setTranscription(Transcription nextTranscription) {
		if (this.transcriptionForThisFrame != null) {
			// could clean up and then set the new transcription. For now
			// the assumption is that the cleaning up has been done or the frame is still empty.
			// throw exception or show an error message pane
			ClientLogger.LOG.warning("There is already a transcription loaded in this frame");
			return;
		}
		if (nextTranscription == null) {
			// could interpret this as a "clean up" request?
			ClientLogger.LOG.warning("There new transcription is null");
			return;
		}
		
		//super.setTranscription(nextTranscription);
		this.transcriptionForThisFrame = nextTranscription;
		initElan();
	}
	
	/**
	 * Creates its own, reduced version of a menu bar.
	 * Might later be changed to enable/disable menus in the "usual" way.
	 *  
	 */
	@Override
	protected void initMenuBar() {
		menuBar = new JMenuBar();
		this.setJMenuBar(menuBar);
        //make menu visible / appear above heavy weight video
        JPopupMenu.setDefaultLightWeightPopupEnabled(false);
        
		MenuAction ma = null;
		
        ma = new MenuAction("Menu.File");
        menuFile = new JMenu(ma);
        menuActions.put("Menu.File", ma);
        menuBar.add(menuFile);
        
        ma = new NewTaSMA(ELANCommandFactory.NEW_DOC, this);
        menuActions.put(ELANCommandFactory.NEW_DOC,ma);
        menuItemFileNew = new JMenuItem(ma);
        menuFile.add(menuItemFileNew);

        ma = new OpenTaSMA(ELANCommandFactory.OPEN_DOC, this);
        menuActions.put(ELANCommandFactory.OPEN_DOC,ma);
        menuItemFileOpen = new JMenuItem(ma);
        menuFile.add(menuItemFileOpen);
        

        ma = new MenuAction(ELANCommandFactory.CLOSE);
        closeMI = new ElanMenuItem(ma, false);
        menuActions.put(ELANCommandFactory.CLOSE, ma);
        menuFile.add(closeMI);
        menuFile.addSeparator();
		
		ma = new MenuAction(ELANCommandFactory.SAVE);
		saveMI = new ElanMenuItem(ma, false);
		menuActions.put(ELANCommandFactory.SAVE, ma);
		menuFile.add(saveMI);
		
		ma = new MenuAction(ELANCommandFactory.SAVE_AS);
		saveAsMI = new ElanMenuItem(ma, false);
		menuActions.put(ELANCommandFactory.SAVE_AS, ma);
		menuFile.add(saveAsMI);
		menuFile.addSeparator();
		
        ma = new ExitTaSMA(ELANCommandFactory.EXIT, this);
        menuActions.put(ELANCommandFactory.EXIT, ma);
        menuItemFileExit = new JMenuItem(ma);
        menuFile.add(menuItemFileExit);

        ma = new MenuAction("Menu.Edit");
        menuActions.put("Menu.Edit", ma);
        menuEdit = new JMenu(ma);
        menuBar.add(menuEdit); 
 
        ma = new MenuAction(ELANCommandFactory.UNDO);
        undoMI = new ElanMenuItem(ma, false);
        menuActions.put(ELANCommandFactory.UNDO, ma);
        menuEdit.add(undoMI);
        ma = new MenuAction(ELANCommandFactory.REDO);
        redoMI = new ElanMenuItem(ma, false);
        menuActions.put(ELANCommandFactory.REDO, ma);
        menuEdit.add(redoMI);
		
        ma = new MenuAction("Menu.Help");
        menuActions.put("Menu.Help", ma);
        menuHelp = new JMenu(ma);
        menuBar.add(menuHelp);

        
        ma = new HelpMA(ELANCommandFactory.HELP, this);
        ma.setEnabled(false);
        menuActions.put(ELANCommandFactory.HELP, ma);
        menuHelp.add(new JMenuItem(ma));		

        updateLocale();
	}
	
	private void initSizeAndLocation() {
		setLocation(0, 0);
		setSize(600, 400);
	}
	
	private void initComponents() {
		//initMenuBar();
		initSizeAndLocation();
		
		updateLocale();
		windowListener = new SLWindowListener();
		addWindowListener(windowListener);
		addWindowFocusListener(windowListener);
		addWindowStateListener(windowListener);
		getContentPane().addComponentListener(new SLComponentListener());
	}
	
	@Override
	public void updateLocale() {
		if (menuBar != null) {
			super.updateLocale();
//			fileMenu.setText(ElanLocale.getString("Menu.File"));
//			newMI.setText(ElanLocale.getString(ELANCommandFactory.NEW_DOC));
//			openMI.setText(ElanLocale.getString(ELANCommandFactory.OPEN_DOC));
//			closeMI.setText(ElanLocale.getString(ELANCommandFactory.CLOSE));
//			saveMI.setText(ElanLocale.getString(ELANCommandFactory.SAVE));
//			saveAsMI.setText(ElanLocale.getString(ELANCommandFactory.SAVE_AS));
//			exitMI.setText(ElanLocale.getString(ELANCommandFactory.EXIT));
//			
//			editMenu.setText(ElanLocale.getString("Menu.Edit"));
//			undoMI.setText(ElanLocale.getString(ELANCommandFactory.UNDO));
//			redoMI.setText(ElanLocale.getString(ELANCommandFactory.REDO));
//			
//			helpMenu.setText(ElanLocale.getString("Menu.Help"));
//			helpMI.setText(ElanLocale.getString(ELANCommandFactory.HELP));
		}
	}
	
	@Override
	protected void initElan() {
		viewerManager = new ViewerManager2((TranscriptionImpl)transcriptionForThisFrame);  
        layoutManager = new ElanLayoutManager(this, viewerManager);
        
        ELANCommandFactory.addDocument(this, viewerManager, layoutManager); 
        layoutManager.changeMode(ElanLayoutManager.TURN_SCENE_MODE);
        //pvMenuManager = new PlayerViewerMenuManager(this, transcriptionForThisFrame);
        //long time = System.currentTimeMillis();
        //System.out.println("B: " + 0);
        //MediaDescriptorUtil.createMediaPlayers((TranscriptionImpl) transcriptionForThisFrame,
        //    transcriptionForThisFrame.getMediaDescriptors());
        MediaDescriptorUtil.createMediaPlayers((TranscriptionImpl) transcriptionForThisFrame,
        		transcriptionForThisFrame.getMediaDescriptors());// for now
        
        //wfvMenuManager = new WaveFormViewerMenuManager(this, transcriptionForThisFrame);

        // if there is a signal viewer its mediafile is the first in the list       
        ArrayList<String> audioPaths = new ArrayList<String>(4);   
       	if (layoutManager.getSignalViewer() != null) {
       		audioPaths.add(layoutManager.getSignalViewer().getMediaPath());
       	}        	
//      
//        if(!menuItemSignalViewer.isSelected()){
//        	if (layoutManager.getSignalViewer() != null) {
//        		layoutManager.remove(layoutManager.getSignalViewer());        		
//        	} 
//        	menuWaveform.setEnabled(false);
//        }
        
        //ArrayList<String> videoPaths = new ArrayList<String>(6);
        // there may be other video files associated with the transcription
//
//        for (MediaDescriptor md : transcriptionForThisFrame.getMediaDescriptors()) {
//    		String path = md.mediaURL;
//    		path = FileUtility.urlToAbsPath(path);
//        	if (md.mimeType.equals(MediaDescriptor.GENERIC_AUDIO_TYPE) ||
//        		md.mimeType.equals(MediaDescriptor.WAV_MIME_TYPE)) {
//    			if (!audioPaths.contains(path)) {
//        		    audioPaths.add(path);
//            	}
//			} else {
//        		if (!videoPaths.contains(path)) {
//        		    videoPaths.add(path);
//        		}
//			}
//        }
//        
//        viewerManager.setAudioPaths(audioPaths); 
//        viewerManager.setVideoPaths(videoPaths);    	

        ElanLocale.addElanLocaleListener(transcriptionForThisFrame, this);
        
        setFrameTitle();
        
        initMenusAndCommands();
        
        //pvMenuManager.initPlayerMenu();
        //wfvMenuManager.initWaveFormViewerMenu();     
        
		layoutManager.changeMode(ElanLayoutManager.TURN_SCENE_MODE);
	
				
		Preferences.addPreferencesListener(transcriptionForThisFrame, layoutManager);
	    Preferences.addPreferencesListener(transcriptionForThisFrame, this);
	    Preferences.notifyListeners(transcriptionForThisFrame);
		
        // a few prefs to load only on load
        loadPreferences();

        // instantiate Viewer components:
        //  use CommandActions for relevant toolbar buttons and combobox menu items
        //  position and size components using LayoutManager
        layoutManager.doLayout();

        //  this sucks but is needed to ensure visible video on the mac
        //viewerManager.getMasterMediaPlayer().setMediaTime(0);
        initialized = true;
	}

	/**
	 * Overridden temporarily? could use the super implementation if preferences
	 * keys are turned into fields etc.
	 */
	@Override
	protected void initFrame() {
		Locale savedLocale = (Locale) Preferences.get("Locale", null);

        if (savedLocale != null) {
            ElanLocale.setLocale(savedLocale);
        }
        // create the initial menu items
        initMenuBar();

        // listen for WindowEvents events on the ElanFrame
        //ElanFrameWindowListener windowList = new ElanFrameWindowListener();
		windowListener = new SLWindowListener();
		addWindowListener(windowListener);
		addWindowFocusListener(windowListener);
		addWindowStateListener(windowListener);
        //addComponentListener(windowListener);

        // require the program to handle the operation in the
        // windowClosing method of a registered WindowListener object.
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

        pack();
        
        //updateShortcutMap(null);

        Dimension d = Preferences.getDimension("FrameSize-SL", transcriptionForThisFrame);
        
        if (d == null) {
        	d = Preferences.getDimension("FrameSize-SL", null);
        }

        // HS June 2010 correct dimension and location if they don't fit on the screen?
        // or make this a user preference?
        Rectangle wRect = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();
        
        if (d != null) {      	
        	if (d.width > wRect.width) {
        		d.setSize(wRect.width, d.height);
        	}
        	if (d.height > wRect.getHeight()) {
        		d.setSize(d.width, wRect.height);
        	}
            setSize(d);
        } else {
        	setSize(800, 600);
        }
        Point p = Preferences.getPoint("FrameLocation-SL", transcriptionForThisFrame);
        
        if (p == null) {
        	p = Preferences.getPoint("FrameLocation-SL", null);
        }

        if (p != null) {
        	// HS 08-2012 only adjust the location if the upper left corner is outside of the 
        	// screen or within some screen inset
        	if (p.x < wRect.x) {
        		p.x = wRect.x;
        	} else if (p.x > wRect.width - WINDOW_POS_MARGIN) {
        		p.x = wRect.width - WINDOW_POS_MARGIN;// or position at left side of the screen?
        	}
//        	int mpx = wRect.width - getWidth();
//        	if (p.x > mpx && mpx > 0) {
//        		p.x = mpx;
//        	}
        	if (p.y < wRect.y) {
        		p.y = wRect.y;
        	} else if (p.y > wRect.height - WINDOW_POS_MARGIN) {
        		p.y = wRect.height - WINDOW_POS_MARGIN;
        	}
//        	int mph = wRect.height - getHeight();
//        	if (p.y > mph && mph > 0) {
//        		p.y = mph;
//        	}
            setLocation(p);
        } else {
        	setLocation((int) ((wRect.getWidth() / 2) - (getWidth() / 2)), 
        			(int) ((wRect.getHeight() / 2) - (getHeight() / 2)));
        }
        // the call to setVisible should be moved to (a new thread created in) 
        // the class that called a constructor....
        setVisible(true);

	}

	@Override
	protected void initMenusAndCommands() {
        //MenuAction ma;

        menuActions.remove(ELANCommandFactory.CLOSE);
        closeMI.setAction(ELANCommandFactory.getCommandAction(transcriptionForThisFrame, 
        		ELANCommandFactory.CLOSE), true);
        menuActions.remove(ELANCommandFactory.SAVE);
        saveMI.setAction(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.SAVE), true);
        menuActions.remove(ELANCommandFactory.SAVE_AS);
        saveAsMI.setAction(ELANCommandFactory.getCommandAction(
                transcriptionForThisFrame, ELANCommandFactory.SAVE_AS), true);
        
        menuActions.remove(ELANCommandFactory.UNDO);
        CommandAction undoCA = ELANCommandFactory.getUndoCA(transcriptionForThisFrame);
        undoMI.setAction(undoCA);

        menuActions.remove(ELANCommandFactory.REDO);
        CommandAction redoCA = ELANCommandFactory.getRedoCA(transcriptionForThisFrame);
        redoMI.setAction(redoCA);
        
        // temporary? fix for the mismatch in what-is-called-when
        localUpdateShortcutMap();
        //updateShortcutMap(ELANCommandFactory.TURNS_SCENE_MODE);
	}
	
	private void localUpdateShortcutMap() {
		// add some additional, temporary? actions
		Action psAction = ELANCommandFactory.getCommandAction(transcriptionForThisFrame, ELANCommandFactory.PLAY_SELECTION);
		// bind Tab to play selection? or to go to next cell?
		KeyStroke TAB_KS = KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0, false);
		String tabPS = "TAB_PLAY_SEL";
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(TAB_KS, tabPS);
		getRootPane().getActionMap().put(tabPS, psAction);
		
		if (undoMI != null && undoMI.getAction() != null) {
			Action undoAC = undoMI.getAction();			
    		KeyStroke ks = ShortcutsUtil.getInstance().getKeyStrokeForAction(ELANCommandFactory.UNDO, 
    				ELANCommandFactory.TURNS_SCENE_MODE);
    		undoAC.putValue(Action.ACCELERATOR_KEY, ks);
    		if (ks != null) {
    			getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(ks, ELANCommandFactory.UNDO);
    			getRootPane().getActionMap().put(ELANCommandFactory.UNDO, undoAC);
    		}
		}

		if (redoMI != null && redoMI.getAction() != null) {
			Action redoAC = redoMI.getAction();
			KeyStroke reKS = ShortcutsUtil.getInstance().getKeyStrokeForAction(ELANCommandFactory.REDO, 
    				ELANCommandFactory.TURNS_SCENE_MODE);
			redoAC.putValue(Action.ACCELERATOR_KEY, reKS);
			if (reKS != null) {
    			getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(reKS, ELANCommandFactory.REDO);
    			getRootPane().getActionMap().put(ELANCommandFactory.REDO, redoAC);
			}
		}
	}

	/**
	 * Temporarily override?
	 * 
	 * @param modeConstant
	 */
	@Override
	public void updateShortcutMap(String modeConstant) {
		// this adds next window - previous window as well, currently not supported in this mode
		super.updateShortcutMap(ELANCommandFactory.TURNS_SCENE_MODE);
		localUpdateShortcutMap();
	}

	/**
	 * Temporarily override?
	 * @param modeConstant
	 */
	@Override
	public void clearShortcutsMap(String modeConstant) {
		super.clearShortcutsMap(ELANCommandFactory.TURNS_SCENE_MODE);
	}

	@Override
	protected void savePreferences() {
		super.savePreferences();
	}

	@Override
	protected void loadPreferences() {
		// super.loadPreferences() also starts a BackUp thread, not sure if that is a problem
		super.loadPreferences();
	}

	@Override
	public void saveAndClose(boolean unregister) {
		super.saveAndClose(false);
	}

	@Override
	public void doClose(boolean unregister) {
		savePreferences();
		
    	if (transcriptionForThisFrame != null) {
  		    layoutManager.isClosing();	// save various preferences in the active mode layout manager
  		    
  		  if (viewerManager != null) {
              viewerManager.cleanUpOnClose();
          } 
      	
  		  if (layoutManager != null) {
			// close all detached frames if any, copied from layout manager	
  			  PlayerLayoutModel model;
  			  List<PlayerLayoutModel> playerList = layoutManager.getPlayerList();
			
  			  for (int i = 0; i < playerList.size(); i++) {
  				  model = playerList.get(i);
  				  if (model.isVisual() && !model.isAttached() && model.detachedFrame != null) {
  					  model.detachedFrame.getContentPane().remove(model.visualComponent);
  					  model.detachedFrame.resetShortcutMaps();
  					  model.detachedFrame.setVisible(false);
  					  model.detachedFrame.dispose();
  					  model.detachedFrame = null;
  				  }
  			  }
  			  this.getContentPane().removeAll();
  			  //this.getContentPane().revalidate();
  			  this.getContentPane().repaint();
  		  }  
      	
          ELANCommandFactory.removeDocument(viewerManager);
          Preferences.removeDocument(transcriptionForThisFrame);
          // remove this frame as locale listener
          ElanLocale.removeElanLocaleListener(transcriptionForThisFrame);            
          
          transcriptionForThisFrame = null;
          viewerManager = null;
          layoutManager = null;
    	} else {
    		// exit the application
    		System.exit(0);
    	}  	
    	
	}
	
	/**
	 * Could reuse or extend the window listener class in ElanFrame2
	 * 
	 *
	 */
	private class SLWindowListener extends WindowAdapter {

		@Override
		public void windowOpened(WindowEvent e) {
			if (layoutManager != null) {
				layoutManager.doLayout();
			}
		}

		@Override
		public void windowClosing(WindowEvent e) {
        	if (transcriptionForThisFrame != null) {
        		// do nothing if this is an empty frame because a new
        		// empty frame would be created
        		checkSaveAndClose();	
        	} else {
        		// 07-2007 changed behaviour: with one empty window, exit the 
        		// application when the close button is pressed
        		doClose(true);
        	}
		}
		
		
	}
	
	private class SLComponentListener extends ComponentAdapter {

		@Override
		public void componentResized(ComponentEvent e) {
			if (layoutManager != null) {
				layoutManager.doLayout();
			}
		}

		@Override
		public void componentShown(ComponentEvent e) {
			if (layoutManager != null) {
				layoutManager.doLayout();
			}
		}
		
	}
}
